
/*  A Bison parser, made from parser.y
 by  GNU Bison version 1.25
  */

#define YYBISON 1  /* Identify Bison output.  */

#define	TOK_ANY	258
#define	TOK_ATTRIBUTE	259
#define	TOK_BOOLEAN	260
#define	TOK_CASE	261
#define	TOK_CHAR	262
#define	TOK_CONST	263
#define	TOK_CONTEXT	264
#define	TOK_DEFAULT	265
#define	TOK_DOUBLE	266
#define	TOK_ENUM	267
#define	TOK_EXCEPTION	268
#define	TOK_FALSE	269
#define	TOK_FIXED	270
#define	TOK_FLOAT	271
#define	TOK_IN	272
#define	TOK_INOUT	273
#define	TOK_INTERFACE	274
#define	TOK_LONG	275
#define	TOK_MODULE	276
#define	TOK_NATIVE	277
#define	TOK_NOSCRIPT	278
#define	TOK_OBJECT	279
#define	TOK_OCTET	280
#define	TOK_ONEWAY	281
#define	TOK_OP_SCOPE	282
#define	TOK_OP_SHL	283
#define	TOK_OP_SHR	284
#define	TOK_OUT	285
#define	TOK_RAISES	286
#define	TOK_READONLY	287
#define	TOK_SEQUENCE	288
#define	TOK_SHORT	289
#define	TOK_STRING	290
#define	TOK_STRUCT	291
#define	TOK_SWITCH	292
#define	TOK_TRUE	293
#define	TOK_TYPECODE	294
#define	TOK_TYPEDEF	295
#define	TOK_UNION	296
#define	TOK_UNSIGNED	297
#define	TOK_VARARGS	298
#define	TOK_VOID	299
#define	TOK_WCHAR	300
#define	TOK_WSTRING	301
#define	TOK_FLOATP	302
#define	TOK_INTEGER	303
#define	TOK_DECLSPEC	304
#define	TOK_PROP_KEY	305
#define	TOK_PROP_VALUE	306
#define	TOK_NATIVE_TYPE	307
#define	TOK_IDENT	308
#define	TOK_SQSTRING	309
#define	TOK_DQSTRING	310
#define	TOK_FIXEDP	311
#define	TOK_CODEFRAG	312

#line 24 "parser.y"

#include <assert.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>
#include "rename.h"
#include "util.h"

#define do_binop(rv,op,a,b)		do {		\
	if (IDL_binop_chktypes (op, a, b))		\
		YYABORT;				\
	if (!(__IDL_flags & IDLF_NO_EVAL_CONST)) {	\
		rv = IDL_binop_eval (op, a, b);		\
		IDL_tree_free (a);			\
		IDL_tree_free (b);			\
		if (!rv) YYABORT;			\
	} else {					\
		rv = IDL_binop_new (op, a, b);		\
	}						\
} while (0)

#define do_unaryop(rv,op,a)		do {		\
	if (IDL_unaryop_chktypes (op, a))		\
		YYABORT;				\
	if (!(__IDL_flags & IDLF_NO_EVAL_CONST)) {	\
		rv = IDL_unaryop_eval (op, a);		\
		IDL_tree_free (a);			\
		if (!rv) YYABORT;			\
	} else {					\
		rv = IDL_unaryop_new (op, a);		\
	}						\
} while (0)

#define assign_props(tree,props)	do {		\
	if (__IDL_flags & IDLF_XPIDL)			\
		IDL_NODE_PROPERTIES (tree) = (props);	\
	else						\
		__IDL_free_properties (props);		\
} while (0)

extern int		yylex				(void);
static IDL_declspec_t	IDL_parse_declspec		(const char *strspec);
static int		IDL_binop_chktypes		(enum IDL_binop op,
							 IDL_tree a,
							 IDL_tree b);
static int		IDL_unaryop_chktypes		(enum IDL_unaryop op,
							 IDL_tree a);
static IDL_tree		IDL_binop_eval			(enum IDL_binop op,
							 IDL_tree a,
							 IDL_tree b);
static IDL_tree		IDL_unaryop_eval		(enum IDL_unaryop op,
							 IDL_tree a);
static IDL_tree		list_start			(IDL_tree a,
							 gboolean filter_null);
static IDL_tree		list_chain			(IDL_tree a,
							 IDL_tree b,
							 gboolean filter_null);
static IDL_tree		zlist_chain			(IDL_tree a,
							 IDL_tree b,
							 gboolean filter_null);
static int		do_token_error			(IDL_tree p,
							 const char *message,
							 gboolean prev);

static GHashTable *				struct_rdht;

#line 94 "parser.y"
typedef union {
	IDL_tree tree;
	struct {
		IDL_tree tree;
		gpointer data;
	} treedata;
	GHashTable *hash_table;
	char *str;
	gboolean boolean;
	IDL_declspec_t declspec;
	IDL_longlong_t integer;
	double floatp;
	enum IDL_unaryop unaryop;
	enum IDL_param_attr paramattr;
} YYSTYPE;
#include <stdio.h>

#ifndef __cplusplus
#ifndef __STDC__
#define const
#endif
#endif



#define	YYFINAL		375
#define	YYFLAG		-32768
#define	YYNTBASE	79

#define YYTRANSLATE(x) ((unsigned)(x) <= 312 ? yytranslate[x] : 212)

static const char yytranslate[] = {     0,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,    73,    68,     2,    63,
    64,    71,    69,    59,    70,     2,    72,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,    62,    58,    75,
    65,    76,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
    77,     2,    78,    67,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,    60,    66,    61,    74,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     1,     2,     3,     4,     5,
     6,     7,     8,     9,    10,    11,    12,    13,    14,    15,
    16,    17,    18,    19,    20,    21,    22,    23,    24,    25,
    26,    27,    28,    29,    30,    31,    32,    33,    34,    35,
    36,    37,    38,    39,    40,    41,    42,    43,    44,    45,
    46,    47,    48,    49,    50,    51,    52,    53,    54,    55,
    56,    57
};

#if YYDEBUG != 0
static const short yyprhs[] = {     0,
     0,     1,     5,     6,     7,     9,    12,    14,    15,    17,
    19,    20,    22,    25,    28,    31,    34,    37,    39,    41,
    43,    46,    53,    59,    61,    63,    65,    66,    78,    84,
    85,    88,    90,    94,    96,    97,   100,   103,   106,   109,
   112,   115,   117,   120,   123,   125,   127,   129,   132,   133,
   139,   142,   144,   146,   148,   150,   152,   154,   156,   158,
   159,   167,   178,   180,   182,   184,   186,   188,   190,   192,
   195,   199,   202,   204,   207,   211,   214,   217,   223,   226,
   233,   234,   237,   238,   240,   245,   247,   249,   251,   253,
   255,   256,   258,   259,   261,   270,   272,   274,   275,   277,
   278,   281,   286,   290,   292,   296,   301,   303,   305,   307,
   309,   310,   312,   313,   315,   320,   325,   327,   329,   331,
   333,   335,   337,   339,   341,   343,   345,   347,   351,   353,
   357,   359,   363,   365,   369,   373,   375,   379,   383,   385,
   389,   393,   397,   400,   402,   404,   406,   408,   410,   412,
   416,   418,   420,   422,   424,   426,   428,   434,   436,   438,
   441,   445,   447,   451,   453,   456,   460,   462,   464,   466,
   468,   470,   472,   474,   476,   478,   480,   482,   484,   486,
   493,   498,   500,   502,   505,   512,   514,   516,   518,   520,
   522,   524,   526,   528,   531,   533,   535,   537,   540,   543,
   547,   549,   551,   553,   555,   557,   559,   561,   566,   568,
   573,   575,   577,   581,   583,   585,   587,   589,   591,   595,
   598,   600,   603,   607,   610,   615,   617,   621,   623,   625,
   627,   629,   630,   632,   634,   636,   638,   640,   644,   646,
   647,   649,   650,   651,   656,   658,   660,   662,   664,   666,
   668,   670,   672,   674,   677,   679
};

static const short yyrhs[] = {    -1,
    80,    82,    81,     0,     0,     0,    87,     0,    82,    87,
     0,    58,     0,     0,    58,     0,    59,     0,     0,   150,
     0,    98,    83,     0,   115,    83,     0,   117,    83,     0,
    91,    83,     0,    89,    83,     0,   208,     0,    86,     0,
    84,     0,   199,    21,     0,    88,   191,    60,    82,    61,
   192,     0,    88,   191,    60,    61,   192,     0,   191,     0,
    24,     0,    39,     0,     0,   199,   200,    19,    90,   192,
    93,    92,    60,    95,    61,   192,     0,   199,   200,    19,
    90,   192,     0,     0,    62,    94,     0,   150,     0,    94,
    85,   150,     0,    96,     0,     0,    96,    97,     0,    98,
    83,     0,   117,    83,     0,   125,    83,     0,   121,    83,
     0,   115,    83,     0,    84,     0,   199,    99,     0,    40,
   101,     0,   105,     0,   107,     0,   149,     0,    22,   181,
     0,     0,    22,   181,    63,   100,    52,     0,   102,   179,
     0,   103,     0,   104,     0,   155,     0,   156,     0,   150,
     0,   105,     0,   107,     0,   149,     0,     0,    36,   190,
    60,   106,   153,    61,   192,     0,    41,   190,    37,    63,
   108,    64,    60,   109,    61,   192,     0,   161,     0,   170,
     0,   172,     0,   149,     0,   150,     0,   110,     0,   111,
     0,   110,   111,     0,   113,   112,    83,     0,   102,   180,
     0,   114,     0,   113,   114,     0,     6,   138,    62,     0,
    10,    62,     0,   199,   116,     0,     8,   137,   189,    65,
   138,     0,   199,   118,     0,    13,   190,    60,   119,    61,
   192,     0,     0,   119,   154,     0,     0,    32,     0,   120,
     4,   122,   183,     0,   155,     0,   177,     0,   178,     0,
   159,     0,   150,     0,     0,    23,     0,     0,    26,     0,
   123,   124,   126,   190,   129,   192,   133,   134,     0,   122,
     0,    44,     0,     0,    43,     0,     0,    59,    43,     0,
    63,   130,   128,    64,     0,    63,   127,    64,     0,   131,
     0,   130,    85,   131,     0,   132,   200,   122,   181,     0,
    17,     0,    30,     0,    18,     0,   122,     0,     0,   135,
     0,     0,   136,     0,    31,    63,    94,    64,     0,     9,
    63,   197,    64,     0,   161,     0,   170,     0,   171,     0,
   172,     0,   158,     0,   177,     0,   178,     0,   160,     0,
   150,     0,   139,     0,   140,     0,   139,    66,   140,     0,
   141,     0,   140,    67,   141,     0,   142,     0,   141,    68,
   142,     0,   143,     0,   142,    29,   143,     0,   142,    28,
   143,     0,   144,     0,   143,    69,   144,     0,   143,    70,
   144,     0,   145,     0,   144,    71,   145,     0,   144,    72,
   145,     0,   144,    73,   145,     0,   146,   147,     0,   147,
     0,    70,     0,    69,     0,    74,     0,   150,     0,   148,
     0,    63,   138,    64,     0,   202,     0,   203,     0,   204,
     0,   205,     0,   206,     0,   207,     0,    12,   189,    60,
   152,    61,     0,   151,     0,   194,     0,    27,   196,     0,
   151,    27,   188,     0,   189,     0,   152,    85,   189,     0,
   154,     0,   153,   154,     0,   102,   179,    83,     0,   158,
     0,   161,     0,   170,     0,   171,     0,   172,     0,   173,
     0,   174,     0,   175,     0,   176,     0,   157,     0,   177,
     0,   178,     0,   159,     0,    33,    75,   103,    59,   198,
    76,     0,    33,    75,   103,    76,     0,    16,     0,    11,
     0,    20,    11,     0,    15,    75,   198,    59,   202,    76,
     0,    15,     0,   162,     0,   166,     0,   163,     0,   164,
     0,   165,     0,    34,     0,    20,     0,    20,    20,     0,
   167,     0,   168,     0,   169,     0,    42,    34,     0,    42,
    20,     0,    42,    20,    20,     0,     7,     0,    45,     0,
     5,     0,    25,     0,     3,     0,    24,     0,    39,     0,
    35,    75,   198,    76,     0,    35,     0,    46,    75,   198,
    76,     0,    46,     0,   180,     0,   179,    85,   180,     0,
   181,     0,   182,     0,   189,     0,   184,     0,   181,     0,
   183,    85,   181,     0,   189,   185,     0,   186,     0,   185,
   186,     0,    77,   198,    78,     0,    50,    51,     0,   187,
    59,    50,    51,     0,    50,     0,   187,    59,    50,     0,
    53,     0,   193,     0,   193,     0,   195,     0,     0,   188,
     0,   188,     0,   188,     0,   188,     0,   203,     0,   197,
    85,   203,     0,   138,     0,     0,    49,     0,     0,     0,
    77,   201,   187,    78,     0,    48,     0,   209,     0,   211,
     0,    56,     0,    47,     0,    38,     0,    14,     0,    57,
     0,   210,     0,   209,   210,     0,    55,     0,    54,     0
};

#endif

#if YYDEBUG != 0
static const short yyrline[] = { 0,
   278,   279,   282,   287,   292,   293,   296,   297,   303,   309,
   310,   316,   322,   323,   324,   325,   326,   327,   328,   329,
   332,   335,   366,   387,   388,   392,   398,   430,   438,   453,
   454,   500,   501,   505,   508,   509,   512,   513,   514,   515,
   516,   517,   520,   528,   529,   530,   531,   532,   533,   542,
   551,   554,   555,   558,   559,   560,   563,   564,   565,   568,
   571,   577,   584,   585,   586,   587,   588,   591,   594,   595,
   598,   602,   605,   606,   609,   610,   613,   621,   628,   636,
   641,   642,   645,   646,   649,   654,   655,   656,   657,   658,
   661,   662,   665,   666,   669,   681,   682,   685,   686,   689,
   690,   693,   700,   706,   707,   711,   720,   721,   722,   723,
   729,   730,   733,   734,   737,   742,   747,   748,   749,   750,
   751,   752,   753,   754,   755,   758,   761,   762,   765,   766,
   769,   770,   773,   774,   775,   778,   779,   780,   783,   784,
   785,   786,   789,   790,   793,   794,   795,   798,   812,   813,
   816,   817,   818,   819,   820,   821,   824,   829,   837,   838,
   839,   866,   867,   871,   872,   875,   888,   889,   890,   891,
   892,   893,   894,   895,   896,   899,   900,   901,   902,   905,
   908,   913,   914,   915,   918,   923,   926,   927,   930,   931,
   932,   935,   938,   941,   944,   945,   946,   949,   952,   955,
   958,   961,   964,   967,   970,   973,   976,   979,   982,   985,
   988,   991,   992,   996,   997,  1000,  1003,  1006,  1007,  1011,
  1015,  1016,  1020,  1025,  1030,  1036,  1040,  1047,  1050,  1058,
  1070,  1086,  1096,  1132,  1148,  1171,  1188,  1189,  1193,  1230,
  1231,  1237,  1238,  1247,  1255,  1258,  1261,  1264,  1267,  1270,
  1271,  1274,  1277,  1278,  1286,  1293
};
#endif


#if YYDEBUG != 0 || defined (YYERROR_VERBOSE)

static const char * const yytname[] = {   "$","error","$undefined.","TOK_ANY",
"TOK_ATTRIBUTE","TOK_BOOLEAN","TOK_CASE","TOK_CHAR","TOK_CONST","TOK_CONTEXT",
"TOK_DEFAULT","TOK_DOUBLE","TOK_ENUM","TOK_EXCEPTION","TOK_FALSE","TOK_FIXED",
"TOK_FLOAT","TOK_IN","TOK_INOUT","TOK_INTERFACE","TOK_LONG","TOK_MODULE","TOK_NATIVE",
"TOK_NOSCRIPT","TOK_OBJECT","TOK_OCTET","TOK_ONEWAY","TOK_OP_SCOPE","TOK_OP_SHL",
"TOK_OP_SHR","TOK_OUT","TOK_RAISES","TOK_READONLY","TOK_SEQUENCE","TOK_SHORT",
"TOK_STRING","TOK_STRUCT","TOK_SWITCH","TOK_TRUE","TOK_TYPECODE","TOK_TYPEDEF",
"TOK_UNION","TOK_UNSIGNED","TOK_VARARGS","TOK_VOID","TOK_WCHAR","TOK_WSTRING",
"TOK_FLOATP","TOK_INTEGER","TOK_DECLSPEC","TOK_PROP_KEY","TOK_PROP_VALUE","TOK_NATIVE_TYPE",
"TOK_IDENT","TOK_SQSTRING","TOK_DQSTRING","TOK_FIXEDP","TOK_CODEFRAG","';'",
"','","'{'","'}'","':'","'('","')'","'='","'|'","'^'","'&'","'+'","'-'","'*'",
"'/'","'%'","'~'","'<'","'>'","'['","']'","specification","init","fini","definition_list",
"check_semicolon","useless_semicolon","check_comma","illegal_ident","definition",
"module_declspec","module","interface_catch_ident","interface","@1","z_inheritance",
"scoped_name_list","interface_body","export_list","export","type_dcl","type_dcl_def",
"@2","type_declarator","type_spec","simple_type_spec","constr_type_spec","struct_type",
"@3","union_type","switch_type_spec","switch_body","case_stmt_list","case_stmt",
"element_spec","case_label_list","case_label","const_dcl","const_dcl_def","except_dcl",
"except_dcl_def","member_zlist","is_readonly","attr_dcl","param_type_spec","is_noscript",
"is_oneway","op_dcl","op_type_spec","is_varargs","is_cvarargs","parameter_dcls",
"param_dcl_list","param_dcl","param_attribute","is_raises_expr","is_context_expr",
"raises_expr","context_expr","const_type","const_exp","or_expr","xor_expr","and_expr",
"shift_expr","add_expr","mult_expr","unary_expr","unary_op","primary_expr","literal",
"enum_type","scoped_name","ns_scoped_name","enumerator_list","member_list","member",
"base_type_spec","template_type_spec","sequence_type","floating_pt_type","fixed_pt_type",
"fixed_pt_const_type","integer_type","signed_int","signed_short_int","signed_long_int",
"signed_longlong_int","unsigned_int","unsigned_short_int","unsigned_long_int",
"unsigned_longlong_int","char_type","wide_char_type","boolean_type","octet_type",
"any_type","object_type","typecode_type","string_type","wide_string_type","declarator_list",
"declarator","simple_declarator","complex_declarator","simple_declarator_list",
"array_declarator","fixed_array_size_list","fixed_array_size","prop_hash","ident",
"new_ident","new_scope","new_or_prev_scope","pop_scope","ns_new_ident","ns_prev_ident",
"cur_ns_new_or_prev_ident","ns_global_ident","string_lit_list","positive_int_const",
"z_declspec","z_props","@4","integer_lit","string_lit","char_lit","fixed_pt_lit",
"floating_pt_lit","boolean_lit","codefrag","dqstring_cat","dqstring","sqstring", NULL
};
#endif

static const short yyr1[] = {     0,
    79,    79,    80,    81,    82,    82,    83,    83,    84,    85,
    85,    86,    87,    87,    87,    87,    87,    87,    87,    87,
    88,    89,    89,    90,    90,    90,    92,    91,    91,    93,
    93,    94,    94,    95,    96,    96,    97,    97,    97,    97,
    97,    97,    98,    99,    99,    99,    99,    99,   100,    99,
   101,   102,   102,   103,   103,   103,   104,   104,   104,   106,
   105,   107,   108,   108,   108,   108,   108,   109,   110,   110,
   111,   112,   113,   113,   114,   114,   115,   116,   117,   118,
   119,   119,   120,   120,   121,   122,   122,   122,   122,   122,
   123,   123,   124,   124,   125,   126,   126,   127,   127,   128,
   128,   129,   129,   130,   130,   131,   132,   132,   132,   132,
   133,   133,   134,   134,   135,   136,   137,   137,   137,   137,
   137,   137,   137,   137,   137,   138,   139,   139,   140,   140,
   141,   141,   142,   142,   142,   143,   143,   143,   144,   144,
   144,   144,   145,   145,   146,   146,   146,   147,   147,   147,
   148,   148,   148,   148,   148,   148,   149,   150,   151,   151,
   151,   152,   152,   153,   153,   154,   155,   155,   155,   155,
   155,   155,   155,   155,   155,   156,   156,   156,   156,   157,
   157,   158,   158,   158,   159,   160,   161,   161,   162,   162,
   162,   163,   164,   165,   166,   166,   166,   167,   168,   169,
   170,   171,   172,   173,   174,   175,   176,   177,   177,   178,
   178,   179,   179,   180,   180,   181,   182,   183,   183,   184,
   185,   185,   186,   187,   187,   187,   187,   188,   189,   190,
   191,   192,   193,   194,   195,   196,   197,   197,   198,   199,
   199,   200,   201,   200,   202,   203,   204,   205,   206,   207,
   207,   208,   209,   209,   210,   211
};

static const short yyr2[] = {     0,
     0,     3,     0,     0,     1,     2,     1,     0,     1,     1,
     0,     1,     2,     2,     2,     2,     2,     1,     1,     1,
     2,     6,     5,     1,     1,     1,     0,    11,     5,     0,
     2,     1,     3,     1,     0,     2,     2,     2,     2,     2,
     2,     1,     2,     2,     1,     1,     1,     2,     0,     5,
     2,     1,     1,     1,     1,     1,     1,     1,     1,     0,
     7,    10,     1,     1,     1,     1,     1,     1,     1,     2,
     3,     2,     1,     2,     3,     2,     2,     5,     2,     6,
     0,     2,     0,     1,     4,     1,     1,     1,     1,     1,
     0,     1,     0,     1,     8,     1,     1,     0,     1,     0,
     2,     4,     3,     1,     3,     4,     1,     1,     1,     1,
     0,     1,     0,     1,     4,     4,     1,     1,     1,     1,
     1,     1,     1,     1,     1,     1,     1,     3,     1,     3,
     1,     3,     1,     3,     3,     1,     3,     3,     1,     3,
     3,     3,     2,     1,     1,     1,     1,     1,     1,     3,
     1,     1,     1,     1,     1,     1,     5,     1,     1,     2,
     3,     1,     3,     1,     2,     3,     1,     1,     1,     1,
     1,     1,     1,     1,     1,     1,     1,     1,     1,     6,
     4,     1,     1,     2,     6,     1,     1,     1,     1,     1,
     1,     1,     1,     2,     1,     1,     1,     2,     2,     3,
     1,     1,     1,     1,     1,     1,     1,     4,     1,     4,
     1,     1,     3,     1,     1,     1,     1,     1,     3,     2,
     1,     2,     3,     2,     4,     1,     3,     1,     1,     1,
     1,     0,     1,     1,     1,     1,     1,     3,     1,     0,
     1,     0,     0,     4,     1,     1,     1,     1,     1,     1,
     1,     1,     1,     2,     1,     1
};

static const short yydefact[] = {     3,
   240,     0,   241,   228,   252,     9,   240,    20,    19,     5,
     0,     8,     8,     8,     8,     8,    12,   158,   234,   159,
   242,    18,   236,   160,     2,     6,   235,     0,   231,     7,
    17,    16,    13,    14,    15,     0,     0,     0,     0,    21,
     0,     0,     0,     0,   243,    43,    45,    46,    77,    79,
    47,     0,   240,   161,   203,   201,   183,   186,   182,   193,
   192,   209,     0,   202,   211,     0,   125,   121,   124,   117,
   187,   189,   190,   191,   188,   195,   196,   197,   118,   119,
   120,   122,   123,   233,     0,   229,     0,   230,    48,   216,
     0,   205,     0,   206,   204,     0,   207,    44,     0,    52,
    53,    57,    58,    59,    56,    54,    55,   176,   167,   179,
   168,   169,   170,   171,   172,   173,   174,   175,   177,   178,
     0,     0,     0,   232,   240,   184,   194,     0,   199,   198,
     0,     0,     0,    81,    49,    60,     0,     0,    51,   212,
   214,   215,   217,   216,     0,   226,     0,    25,    26,   232,
    24,    23,   232,   251,   250,   249,   245,   256,   255,   248,
     0,   146,   145,   147,   239,   126,   127,   129,   131,   133,
   136,   139,     0,   144,   149,   148,     0,   151,   152,   153,
   154,   155,   156,   246,   253,   247,   200,     0,     0,    11,
   162,     0,     0,     0,     0,     0,    10,     0,     0,   220,
   221,     0,   224,     0,   244,    29,    22,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,   143,   208,
   254,   210,    78,   157,     0,   232,     0,    82,    50,     0,
   164,     0,     0,   181,   213,     0,   222,   193,     0,    66,
    67,    63,    64,    65,   227,     0,    27,   150,   128,   130,
   132,   135,   134,   137,   138,   140,   141,   142,   163,    80,
     8,   232,   165,     0,     0,   223,     0,   225,    11,    32,
     0,   166,    61,   185,   180,     0,     0,    35,     0,     0,
     0,    68,    69,     0,    73,    33,     0,    91,     0,    76,
   232,    70,     0,     8,    74,   232,    92,    84,    42,    36,
     8,     8,     8,     0,     8,    93,     8,     0,    75,    62,
    72,    71,    28,    37,    41,    38,     0,    40,    94,     0,
    39,     0,    90,    86,    89,    87,    88,    97,    96,     0,
   218,    85,     0,     0,    98,   232,   219,   107,   109,   108,
    99,   110,     0,    11,   104,   242,   111,   103,    10,     0,
     0,     0,     0,   113,   112,   101,   105,   102,     0,     0,
     0,    95,   114,   106,    11,     0,   115,    11,   237,   116,
     0,   238,     0,     0,     0
};

static const short yydefgoto[] = {   373,
     1,    25,     7,    31,     8,   198,     9,    10,    11,    12,
   150,    13,   271,   247,   269,   287,   288,   300,    14,    46,
   193,    98,   227,   100,   101,   102,   194,   103,   239,   281,
   282,   283,   294,   284,   285,    15,    49,    16,    50,   192,
   304,   305,   342,   306,   320,   307,   330,   343,   351,   336,
   344,   345,   346,   354,   362,   355,   363,    66,   165,   166,
   167,   168,   169,   170,   171,   172,   173,   174,   175,   104,
   176,    18,   190,   230,   228,   106,   107,   108,   109,   110,
    69,   111,    71,    72,    73,    74,    75,    76,    77,    78,
   112,   113,   114,   115,   116,   117,   118,   119,   120,   139,
   140,   141,   142,   332,   143,   200,   201,   147,    19,    90,
    87,    28,   152,    86,    20,    29,    24,   368,   177,    21,
    52,   122,   178,   179,   180,   181,   182,   183,    22,   184,
   185,   186
};

static const short yypact[] = {    67,
    35,     4,-32768,-32768,-32768,-32768,    32,-32768,-32768,-32768,
     4,    11,    11,    11,    11,    11,-32768,   106,-32768,-32768,
    42,-32768,-32768,-32768,-32768,-32768,-32768,    82,-32768,-32768,
-32768,-32768,-32768,-32768,-32768,     4,   714,     4,     4,-32768,
     4,     4,   473,     4,-32768,-32768,-32768,-32768,-32768,-32768,
-32768,   126,   147,-32768,-32768,-32768,-32768,-32768,-32768,    94,
-32768,    78,    36,-32768,    80,     4,-32768,-32768,-32768,-32768,
-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,
-32768,-32768,-32768,-32768,   104,-32768,   117,-32768,   115,-32768,
   120,-32768,   111,-32768,-32768,   122,-32768,-32768,     4,-32768,
-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,
-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,
   142,   131,    62,-32768,   161,-32768,-32768,    93,   165,-32768,
    93,   130,     4,-32768,-32768,-32768,    93,   605,   -14,-32768,
-32768,-32768,-32768,   121,   136,   151,   -30,-32768,-32768,-32768,
-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,
    93,-32768,-32768,-32768,-32768,   140,   145,   141,    69,    66,
    79,-32768,   224,-32768,-32768,-32768,   144,-32768,-32768,-32768,
-32768,-32768,-32768,   162,-32768,-32768,-32768,   148,    93,    16,
-32768,   341,   169,   473,   164,    -8,-32768,     4,    93,   121,
-32768,    53,-32768,   177,-32768,    50,-32768,   167,    93,    93,
    93,    93,    93,    93,    93,    93,    93,    93,-32768,-32768,
-32768,-32768,-32768,-32768,     4,-32768,     4,-32768,-32768,   385,
-32768,   180,    93,-32768,-32768,   155,-32768,   214,   172,-32768,
-32768,-32768,-32768,-32768,   186,     8,-32768,-32768,   145,   141,
    69,    66,    66,    79,    79,-32768,-32768,-32768,-32768,-32768,
   102,-32768,-32768,   173,   174,-32768,   181,-32768,   110,-32768,
   188,-32768,-32768,-32768,-32768,   119,     8,-32768,    93,   178,
   192,   119,-32768,   429,-32768,-32768,   193,   203,   195,-32768,
-32768,-32768,     4,    11,-32768,-32768,-32768,-32768,-32768,-32768,
    11,    11,    11,   254,    11,   233,    11,   234,-32768,-32768,
-32768,-32768,-32768,-32768,-32768,-32768,   693,-32768,-32768,   649,
-32768,     4,-32768,-32768,-32768,-32768,-32768,-32768,-32768,     4,
-32768,   -14,   200,     4,   517,-32768,-32768,-32768,-32768,-32768,
-32768,-32768,   201,    49,-32768,   189,   236,-32768,   226,   561,
   209,   693,   218,   277,-32768,-32768,-32768,-32768,     4,     8,
   227,-32768,-32768,-32768,    57,   162,-32768,    59,-32768,-32768,
   162,-32768,   293,   294,-32768
};

static const short yypgoto[] = {-32768,
-32768,-32768,   242,     7,    14,  -178,-32768,    19,-32768,-32768,
-32768,-32768,-32768,-32768,   -56,-32768,-32768,-32768,    17,-32768,
-32768,-32768,   -29,   168,-32768,   -17,-32768,   -11,-32768,-32768,
-32768,    29,-32768,-32768,    34,    25,-32768,    37,-32768,-32768,
-32768,-32768,  -280,-32768,-32768,-32768,-32768,-32768,-32768,-32768,
-32768,   -24,-32768,-32768,-32768,-32768,-32768,-32768,  -136,-32768,
   114,   118,   109,   -41,   -39,   -59,-32768,   156,-32768,   -20,
    -1,-32768,-32768,-32768,  -156,  -241,-32768,-32768,   295,  -218,
-32768,   -19,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,
   -18,   296,   -13,-32768,-32768,-32768,-32768,   -35,   -28,   103,
  -167,   -38,-32768,-32768,-32768,-32768,   137,-32768,     5,   -33,
   -27,   208,  -123,   -31,-32768,-32768,-32768,-32768,  -103,    48,
    -6,-32768,   113,  -244,-32768,-32768,-32768,-32768,-32768,-32768,
   154,-32768
};


#define	YYLAST		767


static const short yytable[] = {    17,
    51,    82,    89,    47,    85,    17,    23,    88,    83,    48,
    88,   225,    88,    99,    91,    27,   121,    70,    79,    32,
    33,    34,    35,    81,   208,    26,   206,   188,   204,   207,
   235,    -4,   132,   195,     2,    67,   322,   231,   -11,   329,
    54,   105,    84,    84,   197,    84,    84,   205,    84,    37,
   233,    17,   223,    38,    39,   129,     4,    55,     2,    56,
     4,     2,    40,    41,    38,   144,    -1,   234,    30,   130,
    84,   359,   238,   263,   197,   324,   224,    42,   324,     2,
     3,    43,    44,     3,     4,   148,    61,     4,     5,     6,
   277,     5,     6,   324,    63,   236,   212,   213,   325,   191,
   149,   325,   260,    84,   126,     4,   154,   349,   324,   -30,
   324,   246,  -100,   127,     4,   197,   325,   197,    45,     2,
   367,   369,   370,    17,   279,   311,   372,    27,   280,   265,
   155,   325,    36,   325,   214,   215,   105,    84,   273,   156,
   157,    53,   289,    26,   123,     4,   158,   159,   160,   216,
   217,   218,   128,   334,   131,   161,   256,   257,   258,    30,
   197,   162,   163,   133,   144,   350,   164,   310,   197,   -31,
   252,   253,   313,     2,   254,   255,   134,   135,   145,   136,
   146,   240,   242,   243,   187,   137,   277,     2,   244,   371,
   105,   259,   105,   144,   189,     3,   138,   199,   202,     4,
   241,   203,    84,     5,     6,   209,   -83,   124,   211,     3,
  -240,   210,   347,     4,  -240,  -240,   159,     5,     6,   220,
   229,   153,   232,   222,  -240,   297,   245,   157,   105,    84,
   248,    84,   266,   127,   298,   267,   268,   154,  -240,   290,
   276,    37,  -240,  -240,   270,    38,    39,   278,   274,   275,
     2,     3,   291,   296,   293,    41,   309,   317,   319,   144,
     6,   155,   335,   -34,   348,    45,   353,   272,   356,    42,
   156,   157,   358,    43,    44,   286,     4,   158,   159,   160,
   360,   326,   105,   331,   326,   361,   161,    51,   327,   366,
    47,   327,   374,   375,   125,   337,    48,    84,    88,   326,
   312,   299,   333,   365,   301,   196,   327,   314,   315,   316,
   292,   318,   302,   321,   326,   323,   326,   295,   323,   251,
   364,   327,   249,   327,   303,   357,    84,   250,   219,   261,
   151,    68,    80,   323,    84,   308,   237,   221,    84,   352,
     0,     0,     0,    92,   264,    55,     0,    56,   323,     0,
   323,    57,    38,     0,     0,    93,    59,     0,   270,     0,
    60,     0,     0,    84,    94,    95,     0,     2,     0,     0,
     0,     0,     0,    96,    61,    62,    42,     0,     0,    97,
     0,    44,    63,     0,     0,    64,    65,    92,     0,    55,
     0,    56,     0,     4,     0,    57,    38,     0,     0,    93,
    59,   226,     0,     0,    60,     0,     0,     0,    94,    95,
     0,     2,     0,     0,     0,     0,     0,    96,    61,    62,
    42,     0,     0,    97,     0,    44,    63,     0,     0,    64,
    65,    92,     0,    55,   279,    56,     0,     4,   280,    57,
    38,     0,     0,    93,    59,   262,     0,     0,    60,     0,
     0,     0,    94,    95,     0,     2,     0,     0,     0,     0,
     0,    96,    61,    62,    42,     0,     0,    97,     0,    44,
    63,     0,     0,    64,    65,    92,     0,    55,     0,    56,
     0,     4,     0,    57,    38,     0,     0,    93,    59,     0,
     0,     0,    60,     0,     0,     0,    94,    95,     0,     2,
     0,     0,     0,     0,     0,    96,    61,    62,    42,     0,
     0,    97,     0,    44,    63,     0,     0,    64,    65,    92,
     0,    55,     0,    56,     0,     4,     0,    57,     0,     0,
     0,    93,    59,   338,   339,     0,    60,     0,     0,     0,
    94,    95,     0,     2,     0,     0,   340,     0,     0,     0,
    61,    62,     0,     0,     0,    97,     0,     0,    63,   341,
     0,    64,    65,    92,     0,    55,     0,    56,     0,     4,
     0,    57,     0,     0,     0,    93,    59,   338,   339,     0,
    60,     0,     0,     0,    94,    95,     0,     2,     0,     0,
   340,     0,     0,     0,    61,    62,     0,     0,     0,    97,
     0,     0,    63,     0,     0,    64,    65,    92,     0,    55,
     0,    56,     0,     4,     0,    57,     0,     0,     0,    93,
    59,     0,     0,     0,    60,     0,     0,     0,    94,    95,
     0,     2,     0,     0,     0,     0,     0,    96,    61,    62,
     0,     0,     0,    97,     0,     0,    63,     0,     0,    64,
    65,    92,     0,    55,     0,    56,     0,     4,     0,    57,
     0,     0,     0,    93,    59,     0,     0,     0,    60,     0,
     0,     0,    94,    95,     0,     2,     0,     0,     0,     0,
     0,     0,    61,    62,     0,     0,     0,    97,     0,     0,
    63,     0,   328,    64,    65,    92,     0,    55,     0,    56,
     0,     4,     0,    57,     0,     0,     0,    93,    59,     0,
     0,     0,    60,     0,     0,     0,    94,    95,    55,     2,
    56,     0,     0,     0,    57,     0,    61,    62,    58,    59,
     0,    97,     0,    60,    63,     0,     0,    64,    65,     0,
     2,     0,     0,     0,     0,     4,     0,    61,    62,     0,
     0,     0,     0,     0,     0,    63,     0,     0,    64,    65,
     0,     0,     0,     0,     0,     0,     4
};

static const short yycheck[] = {     1,
    21,    37,    41,    21,    38,     7,     2,    39,    37,    21,
    42,   190,    44,    43,    42,    11,    44,    37,    37,    13,
    14,    15,    16,    37,   161,     7,   150,   131,    59,   153,
   198,     0,    66,   137,    27,    37,   317,   194,    53,   320,
    36,    43,    38,    39,    59,    41,    42,    78,    44,     8,
    59,    53,   189,    12,    13,    20,    53,     5,    27,     7,
    53,    27,    21,    22,    12,    99,     0,    76,    58,    34,
    66,   352,    20,   230,    59,   317,    61,    36,   320,    27,
    49,    40,    41,    49,    53,    24,    34,    53,    57,    58,
   269,    57,    58,   335,    42,   199,    28,    29,   317,   133,
    39,   320,   226,    99,    11,    53,    14,    59,   350,    60,
   352,    62,    64,    20,    53,    59,   335,    59,    77,    27,
    64,   366,    64,   125,     6,   293,   371,   123,    10,   233,
    38,   350,    27,   352,    69,    70,   138,   133,   262,    47,
    48,    60,   279,   125,    19,    53,    54,    55,    56,    71,
    72,    73,    75,   332,    75,    63,   216,   217,   218,    58,
    59,    69,    70,    60,   198,   344,    74,   291,    59,    60,
   212,   213,   296,    27,   214,   215,    60,    63,    37,    60,
    50,   202,   202,   202,    20,    75,   365,    27,   202,   368,
   192,   225,   194,   227,    65,    49,    75,    77,    63,    53,
   202,    51,   198,    57,    58,    66,     4,    61,    68,    49,
     8,    67,   336,    53,    12,    13,    55,    57,    58,    76,
    52,    61,    59,    76,    22,    23,    50,    48,   230,   225,
    64,   227,    78,    20,    32,    64,    51,    14,    36,    62,
    60,     8,    40,    41,   246,    12,    13,    60,    76,    76,
    27,    49,    61,    61,   284,    22,    62,     4,    26,   293,
    58,    38,    63,    61,    64,    77,    31,   261,    43,    36,
    47,    48,    64,    40,    41,   277,    53,    54,    55,    56,
    63,   317,   284,   322,   320,     9,    63,   308,   317,    63,
   308,   320,     0,     0,    53,   334,   308,   293,   330,   335,
   294,   288,   330,   360,   288,   138,   335,   301,   302,   303,
   282,   305,   288,   307,   350,   317,   352,   284,   320,   211,
   359,   350,   209,   352,   288,   350,   322,   210,   173,   227,
   123,    37,    37,   335,   330,   288,   200,   184,   334,   346,
    -1,    -1,    -1,     3,   232,     5,    -1,     7,   350,    -1,
   352,    11,    12,    -1,    -1,    15,    16,    -1,   360,    -1,
    20,    -1,    -1,   359,    24,    25,    -1,    27,    -1,    -1,
    -1,    -1,    -1,    33,    34,    35,    36,    -1,    -1,    39,
    -1,    41,    42,    -1,    -1,    45,    46,     3,    -1,     5,
    -1,     7,    -1,    53,    -1,    11,    12,    -1,    -1,    15,
    16,    61,    -1,    -1,    20,    -1,    -1,    -1,    24,    25,
    -1,    27,    -1,    -1,    -1,    -1,    -1,    33,    34,    35,
    36,    -1,    -1,    39,    -1,    41,    42,    -1,    -1,    45,
    46,     3,    -1,     5,     6,     7,    -1,    53,    10,    11,
    12,    -1,    -1,    15,    16,    61,    -1,    -1,    20,    -1,
    -1,    -1,    24,    25,    -1,    27,    -1,    -1,    -1,    -1,
    -1,    33,    34,    35,    36,    -1,    -1,    39,    -1,    41,
    42,    -1,    -1,    45,    46,     3,    -1,     5,    -1,     7,
    -1,    53,    -1,    11,    12,    -1,    -1,    15,    16,    -1,
    -1,    -1,    20,    -1,    -1,    -1,    24,    25,    -1,    27,
    -1,    -1,    -1,    -1,    -1,    33,    34,    35,    36,    -1,
    -1,    39,    -1,    41,    42,    -1,    -1,    45,    46,     3,
    -1,     5,    -1,     7,    -1,    53,    -1,    11,    -1,    -1,
    -1,    15,    16,    17,    18,    -1,    20,    -1,    -1,    -1,
    24,    25,    -1,    27,    -1,    -1,    30,    -1,    -1,    -1,
    34,    35,    -1,    -1,    -1,    39,    -1,    -1,    42,    43,
    -1,    45,    46,     3,    -1,     5,    -1,     7,    -1,    53,
    -1,    11,    -1,    -1,    -1,    15,    16,    17,    18,    -1,
    20,    -1,    -1,    -1,    24,    25,    -1,    27,    -1,    -1,
    30,    -1,    -1,    -1,    34,    35,    -1,    -1,    -1,    39,
    -1,    -1,    42,    -1,    -1,    45,    46,     3,    -1,     5,
    -1,     7,    -1,    53,    -1,    11,    -1,    -1,    -1,    15,
    16,    -1,    -1,    -1,    20,    -1,    -1,    -1,    24,    25,
    -1,    27,    -1,    -1,    -1,    -1,    -1,    33,    34,    35,
    -1,    -1,    -1,    39,    -1,    -1,    42,    -1,    -1,    45,
    46,     3,    -1,     5,    -1,     7,    -1,    53,    -1,    11,
    -1,    -1,    -1,    15,    16,    -1,    -1,    -1,    20,    -1,
    -1,    -1,    24,    25,    -1,    27,    -1,    -1,    -1,    -1,
    -1,    -1,    34,    35,    -1,    -1,    -1,    39,    -1,    -1,
    42,    -1,    44,    45,    46,     3,    -1,     5,    -1,     7,
    -1,    53,    -1,    11,    -1,    -1,    -1,    15,    16,    -1,
    -1,    -1,    20,    -1,    -1,    -1,    24,    25,     5,    27,
     7,    -1,    -1,    -1,    11,    -1,    34,    35,    15,    16,
    -1,    39,    -1,    20,    42,    -1,    -1,    45,    46,    -1,
    27,    -1,    -1,    -1,    -1,    53,    -1,    34,    35,    -1,
    -1,    -1,    -1,    -1,    -1,    42,    -1,    -1,    45,    46,
    -1,    -1,    -1,    -1,    -1,    -1,    53
};
/* -*-C-*-  Note some compilers choke on comments on `#line' lines.  */
#line 3 "/usr/lib/bison.simple"

/* Skeleton output parser for bison,
   Copyright (C) 1984, 1989, 1990 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/* As a special exception, when this file is copied by Bison into a
   Bison output file, you may use that output file without restriction.
   This special exception was added by the Free Software Foundation
   in version 1.24 of Bison.  */

#ifndef alloca
#ifdef __GNUC__
#define alloca __builtin_alloca
#else /* not GNU C.  */
#if (!defined (__STDC__) && defined (sparc)) || defined (__sparc__) || defined (__sparc) || defined (__sgi)
#include <alloca.h>
#else /* not sparc */
#if defined (MSDOS) && !defined (__TURBOC__)
#include <malloc.h>
#else /* not MSDOS, or __TURBOC__ */
#if defined(_AIX)
#include <malloc.h>
 #pragma alloca
#else /* not MSDOS, __TURBOC__, or _AIX */
#ifdef __hpux
#ifdef __cplusplus
extern "C" {
void *alloca (unsigned int);
};
#else /* not __cplusplus */
void *alloca ();
#endif /* not __cplusplus */
#endif /* __hpux */
#endif /* not _AIX */
#endif /* not MSDOS, or __TURBOC__ */
#endif /* not sparc.  */
#endif /* not GNU C.  */
#endif /* alloca not defined.  */

/* This is the parser code that is written into each bison parser
  when the %semantic_parser declaration is not specified in the grammar.
  It was written by Richard Stallman by simplifying the hairy parser
  used when %semantic_parser is specified.  */

/* Note: there must be only one dollar sign in this file.
   It is replaced by the list of actions, each action
   as one case of the switch.  */

#define yyerrok		(yyerrstatus = 0)
#define yyclearin	(yychar = YYEMPTY)
#define YYEMPTY		-2
#define YYEOF		0
#define YYACCEPT	return(0)
#define YYABORT 	return(1)
#define YYERROR		goto yyerrlab1
/* Like YYERROR except do call yyerror.
   This remains here temporarily to ease the
   transition to the new meaning of YYERROR, for GCC.
   Once GCC version 2 has supplanted version 1, this can go.  */
#define YYFAIL		goto yyerrlab
#define YYRECOVERING()  (!!yyerrstatus)
#define YYBACKUP(token, value) \
do								\
  if (yychar == YYEMPTY && yylen == 1)				\
    { yychar = (token), yylval = (value);			\
      yychar1 = YYTRANSLATE (yychar);				\
      YYPOPSTACK;						\
      goto yybackup;						\
    }								\
  else								\
    { yyerror ("syntax error: cannot back up"); YYERROR; }	\
while (0)

#define YYTERROR	1
#define YYERRCODE	256

#ifndef YYPURE
#define YYLEX		yylex()
#endif

#ifdef YYPURE
#ifdef YYLSP_NEEDED
#ifdef YYLEX_PARAM
#define YYLEX		yylex(&yylval, &yylloc, YYLEX_PARAM)
#else
#define YYLEX		yylex(&yylval, &yylloc)
#endif
#else /* not YYLSP_NEEDED */
#ifdef YYLEX_PARAM
#define YYLEX		yylex(&yylval, YYLEX_PARAM)
#else
#define YYLEX		yylex(&yylval)
#endif
#endif /* not YYLSP_NEEDED */
#endif

/* If nonreentrant, generate the variables here */

#ifndef YYPURE

int	yychar;			/*  the lookahead symbol		*/
YYSTYPE	yylval;			/*  the semantic value of the		*/
				/*  lookahead symbol			*/

#ifdef YYLSP_NEEDED
YYLTYPE yylloc;			/*  location data for the lookahead	*/
				/*  symbol				*/
#endif

int yynerrs;			/*  number of parse errors so far       */
#endif  /* not YYPURE */

#if YYDEBUG != 0
int yydebug;			/*  nonzero means print parse trace	*/
/* Since this is uninitialized, it does not stop multiple parsers
   from coexisting.  */
#endif

/*  YYINITDEPTH indicates the initial size of the parser's stacks	*/

#ifndef	YYINITDEPTH
#define YYINITDEPTH 200
#endif

/*  YYMAXDEPTH is the maximum size the stacks can grow to
    (effective only if the built-in stack extension method is used).  */

#if YYMAXDEPTH == 0
#undef YYMAXDEPTH
#endif

#ifndef YYMAXDEPTH
#define YYMAXDEPTH 10000
#endif

/* Prevent warning if -Wstrict-prototypes.  */
#ifdef __GNUC__
int yyparse (void);
#endif

#if __GNUC__ > 1		/* GNU C and GNU C++ define this.  */
#define __yy_memcpy(TO,FROM,COUNT)	__builtin_memcpy(TO,FROM,COUNT)
#else				/* not GNU C or C++ */
#ifndef __cplusplus

/* This is the most reliable way to avoid incompatibilities
   in available built-in functions on various systems.  */
static void
__yy_memcpy (to, from, count)
     char *to;
     char *from;
     int count;
{
  register char *f = from;
  register char *t = to;
  register int i = count;

  while (i-- > 0)
    *t++ = *f++;
}

#else /* __cplusplus */

/* This is the most reliable way to avoid incompatibilities
   in available built-in functions on various systems.  */
static void
__yy_memcpy (char *to, char *from, int count)
{
  register char *f = from;
  register char *t = to;
  register int i = count;

  while (i-- > 0)
    *t++ = *f++;
}

#endif
#endif

#line 196 "/usr/lib/bison.simple"

/* The user can define YYPARSE_PARAM as the name of an argument to be passed
   into yyparse.  The argument should have type void *.
   It should actually point to an object.
   Grammar actions can access the variable by casting it
   to the proper pointer type.  */

#ifdef YYPARSE_PARAM
#ifdef __cplusplus
#define YYPARSE_PARAM_ARG void *YYPARSE_PARAM
#define YYPARSE_PARAM_DECL
#else /* not __cplusplus */
#define YYPARSE_PARAM_ARG YYPARSE_PARAM
#define YYPARSE_PARAM_DECL void *YYPARSE_PARAM;
#endif /* not __cplusplus */
#else /* not YYPARSE_PARAM */
#define YYPARSE_PARAM_ARG
#define YYPARSE_PARAM_DECL
#endif /* not YYPARSE_PARAM */

int
yyparse(YYPARSE_PARAM_ARG)
     YYPARSE_PARAM_DECL
{
  register int yystate;
  register int yyn;
  register short *yyssp;
  register YYSTYPE *yyvsp;
  int yyerrstatus;	/*  number of tokens to shift before error messages enabled */
  int yychar1 = 0;		/*  lookahead token as an internal (translated) token number */

  short	yyssa[YYINITDEPTH];	/*  the state stack			*/
  YYSTYPE yyvsa[YYINITDEPTH];	/*  the semantic value stack		*/

  short *yyss = yyssa;		/*  refer to the stacks thru separate pointers */
  YYSTYPE *yyvs = yyvsa;	/*  to allow yyoverflow to reallocate them elsewhere */

#ifdef YYLSP_NEEDED
  YYLTYPE yylsa[YYINITDEPTH];	/*  the location stack			*/
  YYLTYPE *yyls = yylsa;
  YYLTYPE *yylsp;

#define YYPOPSTACK   (yyvsp--, yyssp--, yylsp--)
#else
#define YYPOPSTACK   (yyvsp--, yyssp--)
#endif

  int yystacksize = YYINITDEPTH;

#ifdef YYPURE
  int yychar;
  YYSTYPE yylval;
  int yynerrs;
#ifdef YYLSP_NEEDED
  YYLTYPE yylloc;
#endif
#endif

  YYSTYPE yyval;		/*  the variable used to return		*/
				/*  semantic values from the action	*/
				/*  routines				*/

  int yylen;

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Starting parse\n");
#endif

  yystate = 0;
  yyerrstatus = 0;
  yynerrs = 0;
  yychar = YYEMPTY;		/* Cause a token to be read.  */

  /* Initialize stack pointers.
     Waste one element of value and location stack
     so that they stay on the same level as the state stack.
     The wasted elements are never initialized.  */

  yyssp = yyss - 1;
  yyvsp = yyvs;
#ifdef YYLSP_NEEDED
  yylsp = yyls;
#endif

/* Push a new state, which is found in  yystate  .  */
/* In all cases, when you get here, the value and location stacks
   have just been pushed. so pushing a state here evens the stacks.  */
yynewstate:

  *++yyssp = yystate;

  if (yyssp >= yyss + yystacksize - 1)
    {
      /* Give user a chance to reallocate the stack */
      /* Use copies of these so that the &'s don't force the real ones into memory. */
      YYSTYPE *yyvs1 = yyvs;
      short *yyss1 = yyss;
#ifdef YYLSP_NEEDED
      YYLTYPE *yyls1 = yyls;
#endif

      /* Get the current used size of the three stacks, in elements.  */
      int size = yyssp - yyss + 1;

#ifdef yyoverflow
      /* Each stack pointer address is followed by the size of
	 the data in use in that stack, in bytes.  */
#ifdef YYLSP_NEEDED
      /* This used to be a conditional around just the two extra args,
	 but that might be undefined if yyoverflow is a macro.  */
      yyoverflow("parser stack overflow",
		 &yyss1, size * sizeof (*yyssp),
		 &yyvs1, size * sizeof (*yyvsp),
		 &yyls1, size * sizeof (*yylsp),
		 &yystacksize);
#else
      yyoverflow("parser stack overflow",
		 &yyss1, size * sizeof (*yyssp),
		 &yyvs1, size * sizeof (*yyvsp),
		 &yystacksize);
#endif

      yyss = yyss1; yyvs = yyvs1;
#ifdef YYLSP_NEEDED
      yyls = yyls1;
#endif
#else /* no yyoverflow */
      /* Extend the stack our own way.  */
      if (yystacksize >= YYMAXDEPTH)
	{
	  yyerror("parser stack overflow");
	  return 2;
	}
      yystacksize *= 2;
      if (yystacksize > YYMAXDEPTH)
	yystacksize = YYMAXDEPTH;
      yyss = (short *) alloca (yystacksize * sizeof (*yyssp));
      __yy_memcpy ((char *)yyss, (char *)yyss1, size * sizeof (*yyssp));
      yyvs = (YYSTYPE *) alloca (yystacksize * sizeof (*yyvsp));
      __yy_memcpy ((char *)yyvs, (char *)yyvs1, size * sizeof (*yyvsp));
#ifdef YYLSP_NEEDED
      yyls = (YYLTYPE *) alloca (yystacksize * sizeof (*yylsp));
      __yy_memcpy ((char *)yyls, (char *)yyls1, size * sizeof (*yylsp));
#endif
#endif /* no yyoverflow */

      yyssp = yyss + size - 1;
      yyvsp = yyvs + size - 1;
#ifdef YYLSP_NEEDED
      yylsp = yyls + size - 1;
#endif

#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Stack size increased to %d\n", yystacksize);
#endif

      if (yyssp >= yyss + yystacksize - 1)
	YYABORT;
    }

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Entering state %d\n", yystate);
#endif

  goto yybackup;
 yybackup:

/* Do appropriate processing given the current state.  */
/* Read a lookahead token if we need one and don't already have one.  */
/* yyresume: */

  /* First try to decide what to do without reference to lookahead token.  */

  yyn = yypact[yystate];
  if (yyn == YYFLAG)
    goto yydefault;

  /* Not known => get a lookahead token if don't already have one.  */

  /* yychar is either YYEMPTY or YYEOF
     or a valid token in external form.  */

  if (yychar == YYEMPTY)
    {
#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Reading a token: ");
#endif
      yychar = YYLEX;
    }

  /* Convert token to internal form (in yychar1) for indexing tables with */

  if (yychar <= 0)		/* This means end of input. */
    {
      yychar1 = 0;
      yychar = YYEOF;		/* Don't call YYLEX any more */

#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Now at end of input.\n");
#endif
    }
  else
    {
      yychar1 = YYTRANSLATE(yychar);

#if YYDEBUG != 0
      if (yydebug)
	{
	  fprintf (stderr, "Next token is %d (%s", yychar, yytname[yychar1]);
	  /* Give the individual parser a way to print the precise meaning
	     of a token, for further debugging info.  */
#ifdef YYPRINT
	  YYPRINT (stderr, yychar, yylval);
#endif
	  fprintf (stderr, ")\n");
	}
#endif
    }

  yyn += yychar1;
  if (yyn < 0 || yyn > YYLAST || yycheck[yyn] != yychar1)
    goto yydefault;

  yyn = yytable[yyn];

  /* yyn is what to do for this token type in this state.
     Negative => reduce, -yyn is rule number.
     Positive => shift, yyn is new state.
       New state is final state => don't bother to shift,
       just return success.
     0, or most negative number => error.  */

  if (yyn < 0)
    {
      if (yyn == YYFLAG)
	goto yyerrlab;
      yyn = -yyn;
      goto yyreduce;
    }
  else if (yyn == 0)
    goto yyerrlab;

  if (yyn == YYFINAL)
    YYACCEPT;

  /* Shift the lookahead token.  */

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Shifting token %d (%s), ", yychar, yytname[yychar1]);
#endif

  /* Discard the token being shifted unless it is eof.  */
  if (yychar != YYEOF)
    yychar = YYEMPTY;

  *++yyvsp = yylval;
#ifdef YYLSP_NEEDED
  *++yylsp = yylloc;
#endif

  /* count tokens shifted since error; after three, turn off error status.  */
  if (yyerrstatus) yyerrstatus--;

  yystate = yyn;
  goto yynewstate;

/* Do the default action for the current state.  */
yydefault:

  yyn = yydefact[yystate];
  if (yyn == 0)
    goto yyerrlab;

/* Do a reduction.  yyn is the number of a rule to reduce with.  */
yyreduce:
  yylen = yyr2[yyn];
  if (yylen > 0)
    yyval = yyvsp[1-yylen]; /* implement default value of the action */

#if YYDEBUG != 0
  if (yydebug)
    {
      int i;

      fprintf (stderr, "Reducing via rule %d (line %d), ",
	       yyn, yyrline[yyn]);

      /* Print the symbols being reduced, and their result.  */
      for (i = yyprhs[yyn]; yyrhs[i] > 0; i++)
	fprintf (stderr, "%s ", yytname[yyrhs[i]]);
      fprintf (stderr, " -> %s\n", yytname[yyr1[yyn]]);
    }
#endif


  switch (yyn) {

case 1:
#line 278 "parser.y"
{ yyerror ("Empty file"); YYABORT; ;
    break;}
case 2:
#line 279 "parser.y"
{ __IDL_root = yyvsp[-1].tree; ;
    break;}
case 3:
#line 282 "parser.y"
{
	struct_rdht = g_hash_table_new (g_direct_hash, g_direct_equal);
;
    break;}
case 4:
#line 287 "parser.y"
{
	g_hash_table_destroy (struct_rdht);
;
    break;}
case 5:
#line 292 "parser.y"
{ yyval.tree = list_start (yyvsp[0].tree, TRUE); ;
    break;}
case 6:
#line 293 "parser.y"
{ yyval.tree = list_chain (yyvsp[-1].tree, yyvsp[0].tree, TRUE); ;
    break;}
case 8:
#line 297 "parser.y"
{
	if (do_token_error (yyvsp[0].tree, "Missing semicolon after", TRUE))
		YYABORT;
;
    break;}
case 9:
#line 303 "parser.y"
{
	yyerror ("Dangling semicolon has no effect");
	yyval.tree = NULL;
;
    break;}
case 11:
#line 310 "parser.y"
{
	if (do_token_error (yyvsp[0].tree, "Missing comma after", TRUE))
		YYABORT;
;
    break;}
case 12:
#line 316 "parser.y"
{
	assert (IDL_NODE_UP (yyvsp[0].tree) != NULL);
	do_token_error (IDL_NODE_UP (yyvsp[0].tree), "Illegal context for", FALSE);
;
    break;}
case 22:
#line 337 "parser.y"
{
	IDL_tree module;

	if (IDL_NODE_UP (yyvsp[-4].tree) != NULL &&
	    IDL_NODE_TYPE (IDL_NODE_UP (yyvsp[-4].tree)) != IDLN_MODULE) {
		do_token_error (IDL_NODE_UP (yyvsp[-4].tree), "Module definition conflicts with", FALSE);
		IDL_tree_error (yyvsp[-4].tree, "Previous declaration");
		YYABORT;
	}

	if (__IDL_flags & IDLF_COMBINE_REOPENED_MODULES) {
		if (IDL_NODE_UP (yyvsp[-4].tree) == NULL)
			module = IDL_module_new (yyvsp[-4].tree, yyvsp[-2].tree);
		else {
			module = IDL_NODE_UP (yyvsp[-4].tree);
			
			IDL_MODULE (module).definition_list =
				IDL_list_concat (IDL_MODULE (module).definition_list, yyvsp[-2].tree);

			module = NULL;
		}
	} else
		module = IDL_module_new (yyvsp[-4].tree, yyvsp[-2].tree);

	yyval.tree = module;
	IDL_NODE_DECLSPEC (yyval.tree) = yyvsp[-5].declspec;	
	if (__IDL_inhibits > 0)
		IDL_NODE_DECLSPEC (yyval.tree) |= IDLF_DECLSPEC_EXIST | IDLF_DECLSPEC_INHIBIT;
;
    break;}
case 23:
#line 367 "parser.y"
{
	if (IDL_NODE_UP (yyvsp[-3].tree) != NULL &&
	    IDL_NODE_TYPE (IDL_NODE_UP (yyvsp[-3].tree)) != IDLN_MODULE) {
		do_token_error (IDL_NODE_UP (yyvsp[-3].tree), "Module definition conflicts with", FALSE);
		IDL_tree_error (yyvsp[-3].tree, "Previous declaration");
		YYABORT;
	}
	yywarningv (IDL_WARNING1,
		   "Empty module declaration `%s' is not legal IDL",
		   IDL_IDENT (yyvsp[-3].tree).str);

	yyval.tree = IDL_NODE_UP (yyvsp[-3].tree) ? NULL : IDL_module_new (yyvsp[-3].tree, NULL);
	if (yyval.tree) {
		IDL_NODE_DECLSPEC (yyval.tree) = yyvsp[-4].declspec;
		if (__IDL_inhibits > 0)
			IDL_NODE_DECLSPEC (yyval.tree) |= IDLF_DECLSPEC_EXIST | IDLF_DECLSPEC_INHIBIT;
	}
;
    break;}
case 25:
#line 388 "parser.y"
{
	yyerror ("Interfaces cannot be named `Object'");
	YYABORT;
;
    break;}
case 26:
#line 392 "parser.y"
{
	yyerror ("Interfaces cannot be named `TypeCode'");
	YYABORT;
;
    break;}
case 27:
#line 403 "parser.y"
{
	assert (yyvsp[-2].tree != NULL);
	assert (IDL_NODE_TYPE (yyvsp[-2].tree) == IDLN_IDENT);
	assert (IDL_IDENT_TO_NS (yyvsp[-2].tree) != NULL);
	assert (IDL_NODE_TYPE (IDL_IDENT_TO_NS (yyvsp[-2].tree)) == IDLN_GENTREE);
	if (IDL_NODE_UP (yyvsp[-2].tree) != NULL &&
	    IDL_NODE_TYPE (IDL_NODE_UP (yyvsp[-2].tree)) != IDLN_INTERFACE &&
	    IDL_NODE_TYPE (IDL_NODE_UP (yyvsp[-2].tree)) != IDLN_FORWARD_DCL) {
		do_token_error (IDL_NODE_UP (yyvsp[-2].tree), "Interface definition conflicts with", FALSE);
		IDL_tree_error (yyvsp[-2].tree, "Previous declaration");
		YYABORT;
	} else if (IDL_NODE_UP (yyvsp[-2].tree) != NULL &&
		   IDL_NODE_TYPE (IDL_NODE_UP (yyvsp[-2].tree)) != IDLN_FORWARD_DCL) {
		yyerrorv ("Cannot redeclare interface `%s'", IDL_IDENT (yyvsp[-2].tree).str);
		IDL_tree_error (yyvsp[-2].tree, "Previous declaration of interface `%s'", IDL_IDENT (yyvsp[-2].tree).str);
		YYABORT;
	} else if (IDL_NODE_UP (yyvsp[-2].tree) != NULL &&
		   IDL_NODE_TYPE (IDL_NODE_UP (yyvsp[-2].tree)) == IDLN_FORWARD_DCL) {
		yyvsp[-2].tree->_file = __IDL_cur_filename;
		yyvsp[-2].tree->_line = __IDL_cur_line;
	}
	IDL_GENTREE (IDL_IDENT_TO_NS (yyvsp[-2].tree))._import = yyvsp[0].tree;
	IDL_ns_push_scope (__IDL_root_ns, IDL_IDENT_TO_NS (yyvsp[-2].tree));
	if (IDL_ns_check_for_ambiguous_inheritance (yyvsp[-2].tree, yyvsp[0].tree))
		__IDL_is_okay = FALSE;
;
    break;}
case 28:
#line 431 "parser.y"
{
 	yyval.tree = IDL_interface_new (yyvsp[-7].tree, yyvsp[-5].tree, yyvsp[-2].tree);
	IDL_NODE_DECLSPEC (yyval.tree) = yyvsp[-10].declspec;
	if (__IDL_inhibits > 0)
		IDL_NODE_DECLSPEC (yyval.tree) |= IDLF_DECLSPEC_EXIST | IDLF_DECLSPEC_INHIBIT;
	assign_props (yyval.tree, yyvsp[-9].hash_table);
;
    break;}
case 29:
#line 442 "parser.y"
{
	if (yyvsp[-3].hash_table) yywarningv (IDL_WARNING1,
			    "Ignoring properties for forward declaration `%s'",
			    IDL_IDENT (yyvsp[-1].tree));
	yyval.tree = IDL_forward_dcl_new (yyvsp[-1].tree);
	IDL_NODE_DECLSPEC (yyval.tree) = yyvsp[-4].declspec;
	if (__IDL_inhibits > 0)
		IDL_NODE_DECLSPEC (yyval.tree) |= IDLF_DECLSPEC_EXIST | IDLF_DECLSPEC_INHIBIT;
;
    break;}
case 30:
#line 453 "parser.y"
{ yyval.tree = NULL; ;
    break;}
case 31:
#line 454 "parser.y"
{
	GHashTable *table = g_hash_table_new (g_direct_hash, g_direct_equal);
	gboolean die = FALSE;
	IDL_tree p = yyvsp[0].tree;

	assert (IDL_NODE_TYPE (p) == IDLN_LIST);
	for (; p != NULL && !die; p = IDL_LIST (p).next) {
		assert (IDL_LIST (p).data != NULL);
		assert (IDL_NODE_TYPE (IDL_LIST (p).data) == IDLN_IDENT);

		if (g_hash_table_lookup_extended (table, IDL_LIST (p).data, NULL, NULL)) {
			char *s = IDL_ns_ident_to_qstring (IDL_LIST (p).data, "::", 0);
			yyerrorv ("Cannot inherit from interface `%s' more than once", s);
			free (s);
			die = TRUE;
			break;
		} else
			g_hash_table_insert (table, IDL_LIST (p).data, NULL);

		if (IDL_NODE_TYPE (IDL_NODE_UP (IDL_LIST (p).data)) == IDLN_FORWARD_DCL) {
			char *s = IDL_ns_ident_to_qstring (IDL_LIST (p).data, "::", 0);
			yyerrorv ("Incomplete definition of interface `%s'", s);
			IDL_tree_error (IDL_LIST (p).data,
					"Previous forward declaration of `%s'", s);
			free (s);
			die = TRUE;
		}
		else if (IDL_NODE_TYPE (IDL_NODE_UP (IDL_LIST (p).data)) != IDLN_INTERFACE) {
			char *s = IDL_ns_ident_to_qstring (IDL_LIST (p).data, "::", 0);
			yyerrorv ("`%s' is not an interface", s);
			IDL_tree_error (IDL_LIST (p).data,
					"Previous declaration of `%s'", s);
			free (s);
			die = TRUE;
		}
	}

	g_hash_table_destroy (table);

	if (die)
		YYABORT;

	yyval.tree = yyvsp[0].tree;
;
    break;}
case 32:
#line 500 "parser.y"
{ yyval.tree = list_start (yyvsp[0].tree, TRUE); ;
    break;}
case 33:
#line 502 "parser.y"
{ yyval.tree = list_chain (yyvsp[-2].tree, yyvsp[0].tree, TRUE); ;
    break;}
case 35:
#line 508 "parser.y"
{ yyval.tree = NULL; ;
    break;}
case 36:
#line 509 "parser.y"
{ yyval.tree = zlist_chain (yyvsp[-1].tree, yyvsp[0].tree, TRUE); ;
    break;}
case 43:
#line 520 "parser.y"
{
	yyval.tree = yyvsp[0].tree;
	IDL_NODE_DECLSPEC (yyval.tree) = yyvsp[-1].declspec;
	if (__IDL_inhibits > 0)
 		IDL_NODE_DECLSPEC (yyval.tree) |= IDLF_DECLSPEC_EXIST | IDLF_DECLSPEC_INHIBIT;
;
    break;}
case 44:
#line 528 "parser.y"
{ yyval.tree = yyvsp[0].tree; ;
    break;}
case 48:
#line 532 "parser.y"
{ yyval.tree = IDL_native_new (yyvsp[0].tree); ;
    break;}
case 49:
#line 534 "parser.y"
{
	/* Enable native type scanning */
	if (__IDL_flags & IDLF_XPIDL)
		__IDL_flagsi |= IDLFP_XPIDL_NATIVE;
	else {
		yyerror ("XPIDL syntax not enabled, cannot specify native extension");
		YYABORT;
	}
;
    break;}
case 50:
#line 542 "parser.y"
{
	/* Disable native type scanning */
	if (__IDL_flags & IDLF_XPIDL)
		__IDL_flagsi &= ~IDLFP_XPIDL_NATIVE;
	yyval.tree = IDL_native_new (yyvsp[-3].tree);
	IDL_NATIVE (yyval.tree).user_type = yyvsp[0].str;
;
    break;}
case 51:
#line 551 "parser.y"
{ yyval.tree = IDL_type_dcl_new (yyvsp[-1].tree, yyvsp[0].tree); ;
    break;}
case 60:
#line 568 "parser.y"
{
	g_hash_table_insert (struct_rdht, yyvsp[-1].tree, yyvsp[-1].tree);
;
    break;}
case 61:
#line 571 "parser.y"
{
	yyval.tree = IDL_type_struct_new (yyvsp[-5].tree, yyvsp[-2].tree);
	g_hash_table_remove (struct_rdht, yyvsp[-5].tree);
;
    break;}
case 62:
#line 581 "parser.y"
{ yyval.tree = IDL_type_union_new (yyvsp[-8].tree, yyvsp[-5].tree, yyvsp[-2].tree); ;
    break;}
case 69:
#line 594 "parser.y"
{ yyval.tree = list_start (yyvsp[0].tree, TRUE); ;
    break;}
case 70:
#line 595 "parser.y"
{ yyval.tree = list_chain (yyvsp[-1].tree, yyvsp[0].tree, TRUE); ;
    break;}
case 71:
#line 599 "parser.y"
{ yyval.tree = IDL_case_stmt_new (yyvsp[-2].tree, yyvsp[-1].tree); ;
    break;}
case 72:
#line 602 "parser.y"
{ yyval.tree = IDL_member_new (yyvsp[-1].tree, list_start (yyvsp[0].tree, TRUE)); ;
    break;}
case 73:
#line 605 "parser.y"
{ yyval.tree = list_start (yyvsp[0].tree, FALSE); ;
    break;}
case 74:
#line 606 "parser.y"
{ yyval.tree = list_chain (yyvsp[-1].tree, yyvsp[0].tree, FALSE); ;
    break;}
case 75:
#line 609 "parser.y"
{ yyval.tree = yyvsp[-1].tree; ;
    break;}
case 76:
#line 610 "parser.y"
{ yyval.tree = NULL; ;
    break;}
case 77:
#line 613 "parser.y"
{
	yyval.tree = yyvsp[0].tree;
	IDL_NODE_DECLSPEC (yyval.tree) = yyvsp[-1].declspec;
	if (__IDL_inhibits > 0)
 		IDL_NODE_DECLSPEC (yyval.tree) |= IDLF_DECLSPEC_EXIST | IDLF_DECLSPEC_INHIBIT;
;
    break;}
case 78:
#line 622 "parser.y"
{
	yyval.tree = IDL_const_dcl_new (yyvsp[-3].tree, yyvsp[-2].tree, yyvsp[0].tree);
	/* Should probably do some type checking here... */
;
    break;}
case 79:
#line 628 "parser.y"
{
	yyval.tree = yyvsp[0].tree;
	IDL_NODE_DECLSPEC (yyval.tree) = yyvsp[-1].declspec;
	if (__IDL_inhibits > 0)
 		IDL_NODE_DECLSPEC (yyval.tree) |= IDLF_DECLSPEC_EXIST | IDLF_DECLSPEC_INHIBIT;
;
    break;}
case 80:
#line 638 "parser.y"
{ yyval.tree = IDL_except_dcl_new (yyvsp[-4].tree, yyvsp[-2].tree); ;
    break;}
case 81:
#line 641 "parser.y"
{ yyval.tree = NULL; ;
    break;}
case 82:
#line 642 "parser.y"
{ yyval.tree = zlist_chain (yyvsp[-1].tree, yyvsp[0].tree, TRUE); ;
    break;}
case 83:
#line 645 "parser.y"
{ yyval.boolean = FALSE; ;
    break;}
case 84:
#line 646 "parser.y"
{ yyval.boolean = TRUE; ;
    break;}
case 85:
#line 651 "parser.y"
{ yyval.tree = IDL_attr_dcl_new (yyvsp[-3].boolean, yyvsp[-1].tree, yyvsp[0].tree); ;
    break;}
case 91:
#line 661 "parser.y"
{ yyval.boolean = FALSE; ;
    break;}
case 92:
#line 662 "parser.y"
{ yyval.boolean = TRUE; ;
    break;}
case 93:
#line 665 "parser.y"
{ yyval.boolean = FALSE; ;
    break;}
case 94:
#line 666 "parser.y"
{ yyval.boolean = TRUE; ;
    break;}
case 95:
#line 674 "parser.y"
{
	yyval.tree = IDL_op_dcl_new (yyvsp[-6].boolean, yyvsp[-5].tree, yyvsp[-4].tree, yyvsp[-3].treedata.tree, yyvsp[-1].tree, yyvsp[0].tree);
	IDL_OP_DCL (yyval.tree).f_noscript = yyvsp[-7].boolean;
	IDL_OP_DCL (yyval.tree).f_varargs = (gboolean) GPOINTER_TO_INT (yyvsp[-3].treedata.data);
;
    break;}
case 97:
#line 682 "parser.y"
{ yyval.tree = NULL; ;
    break;}
case 98:
#line 685 "parser.y"
{ yyval.boolean = FALSE; ;
    break;}
case 99:
#line 686 "parser.y"
{ yyval.boolean = TRUE; ;
    break;}
case 100:
#line 689 "parser.y"
{ yyval.boolean = FALSE; ;
    break;}
case 101:
#line 690 "parser.y"
{ yyval.boolean = TRUE; ;
    break;}
case 102:
#line 696 "parser.y"
{
	yyval.treedata.tree = yyvsp[-2].tree;
	yyval.treedata.data = GINT_TO_POINTER (yyvsp[-1].boolean);
;
    break;}
case 103:
#line 700 "parser.y"
{
	yyval.treedata.tree = NULL;
	yyval.treedata.data = GINT_TO_POINTER (yyvsp[-1].boolean);
;
    break;}
case 104:
#line 706 "parser.y"
{ yyval.tree = list_start (yyvsp[0].tree, TRUE); ;
    break;}
case 105:
#line 708 "parser.y"
{ yyval.tree = list_chain (yyvsp[-2].tree, yyvsp[0].tree, TRUE); ;
    break;}
case 106:
#line 714 "parser.y"
{
	yyval.tree = IDL_param_dcl_new (yyvsp[-3].paramattr, yyvsp[-1].tree, yyvsp[0].tree);
	assign_props (yyval.tree, yyvsp[-2].hash_table);
;
    break;}
case 107:
#line 720 "parser.y"
{ yyval.paramattr = IDL_PARAM_IN; ;
    break;}
case 108:
#line 721 "parser.y"
{ yyval.paramattr = IDL_PARAM_OUT; ;
    break;}
case 109:
#line 722 "parser.y"
{ yyval.paramattr = IDL_PARAM_INOUT; ;
    break;}
case 110:
#line 723 "parser.y"
{
	yyerrorv ("Missing direction attribute (in, out, inout) before parameter");
	IDL_tree_free (yyvsp[0].tree);
;
    break;}
case 111:
#line 729 "parser.y"
{ yyval.tree = NULL; ;
    break;}
case 113:
#line 733 "parser.y"
{ yyval.tree = NULL; ;
    break;}
case 115:
#line 739 "parser.y"
{ yyval.tree = yyvsp[-1].tree; ;
    break;}
case 116:
#line 744 "parser.y"
{ yyval.tree = yyvsp[-1].tree; ;
    break;}
case 128:
#line 762 "parser.y"
{ do_binop (yyval.tree, IDL_BINOP_OR, yyvsp[-2].tree, yyvsp[0].tree); ;
    break;}
case 130:
#line 766 "parser.y"
{ do_binop (yyval.tree, IDL_BINOP_XOR, yyvsp[-2].tree, yyvsp[0].tree); ;
    break;}
case 132:
#line 770 "parser.y"
{ do_binop (yyval.tree, IDL_BINOP_AND, yyvsp[-2].tree, yyvsp[0].tree); ;
    break;}
case 134:
#line 774 "parser.y"
{ do_binop (yyval.tree, IDL_BINOP_SHR, yyvsp[-2].tree, yyvsp[0].tree); ;
    break;}
case 135:
#line 775 "parser.y"
{ do_binop (yyval.tree, IDL_BINOP_SHL, yyvsp[-2].tree, yyvsp[0].tree); ;
    break;}
case 137:
#line 779 "parser.y"
{ do_binop (yyval.tree, IDL_BINOP_ADD, yyvsp[-2].tree, yyvsp[0].tree); ;
    break;}
case 138:
#line 780 "parser.y"
{ do_binop (yyval.tree, IDL_BINOP_SUB, yyvsp[-2].tree, yyvsp[0].tree); ;
    break;}
case 140:
#line 784 "parser.y"
{ do_binop (yyval.tree, IDL_BINOP_MULT, yyvsp[-2].tree, yyvsp[0].tree); ;
    break;}
case 141:
#line 785 "parser.y"
{ do_binop (yyval.tree, IDL_BINOP_DIV, yyvsp[-2].tree, yyvsp[0].tree); ;
    break;}
case 142:
#line 786 "parser.y"
{ do_binop (yyval.tree, IDL_BINOP_MOD, yyvsp[-2].tree, yyvsp[0].tree); ;
    break;}
case 143:
#line 789 "parser.y"
{ do_unaryop (yyval.tree, yyvsp[-1].unaryop, yyvsp[0].tree); ;
    break;}
case 145:
#line 793 "parser.y"
{ yyval.unaryop = IDL_UNARYOP_MINUS; ;
    break;}
case 146:
#line 794 "parser.y"
{ yyval.unaryop = IDL_UNARYOP_PLUS; ;
    break;}
case 147:
#line 795 "parser.y"
{ yyval.unaryop = IDL_UNARYOP_COMPLEMENT; ;
    break;}
case 148:
#line 798 "parser.y"
{
	IDL_tree p, literal;
	
	assert (IDL_NODE_TYPE (yyvsp[0].tree) == IDLN_IDENT);

	p = IDL_NODE_UP (yyvsp[0].tree);
	
	if ((literal = IDL_resolve_const_exp (yyvsp[0].tree, IDLN_ANY))) {
		++IDL_NODE_REFS (literal);
		yyval.tree = literal;
		IDL_tree_free (yyvsp[0].tree);
	} else
		yyval.tree = yyvsp[0].tree;
;
    break;}
case 150:
#line 813 "parser.y"
{ yyval.tree = yyvsp[-1].tree; ;
    break;}
case 157:
#line 826 "parser.y"
{ yyval.tree = IDL_type_enum_new (yyvsp[-3].tree, yyvsp[-1].tree); ;
    break;}
case 158:
#line 829 "parser.y"
{
	assert (yyvsp[0].tree != NULL);
	assert (IDL_NODE_TYPE (yyvsp[0].tree) == IDLN_GENTREE);
	assert (IDL_NODE_TYPE (IDL_GENTREE (yyvsp[0].tree).data) == IDLN_IDENT);
	yyval.tree = IDL_GENTREE (yyvsp[0].tree).data;
;
    break;}
case 160:
#line 838 "parser.y"
{ yyval.tree = yyvsp[0].tree; ;
    break;}
case 161:
#line 840 "parser.y"
{
	IDL_tree p;

	assert (IDL_NODE_TYPE (yyvsp[-2].tree) == IDLN_GENTREE);
	assert (IDL_NODE_TYPE (yyvsp[0].tree) == IDLN_IDENT);

#ifdef YYDEBUG
	if (yydebug)
		printf ("looking in %s\n", IDL_IDENT (IDL_GENTREE (yyvsp[-2].tree).data).str);
#endif

	if ((p = IDL_ns_lookup_this_scope (__IDL_root_ns, yyvsp[-2].tree, yyvsp[0].tree, NULL)) == NULL) {
#ifdef YYDEBUG
		if (yydebug)
			printf ("'%s'\n", IDL_IDENT (yyvsp[0].tree).str);
#endif
		yyerrorv ("`%s' undeclared identifier", IDL_IDENT (yyvsp[0].tree).str);
		IDL_tree_free (yyvsp[0].tree);
		YYABORT;
	}
	IDL_tree_free (yyvsp[0].tree);
	++IDL_NODE_REFS (IDL_GENTREE (p).data);
	yyval.tree = p;
;
    break;}
case 162:
#line 866 "parser.y"
{ yyval.tree = list_start (yyvsp[0].tree, TRUE); ;
    break;}
case 163:
#line 868 "parser.y"
{ yyval.tree = list_chain (yyvsp[-2].tree, yyvsp[0].tree, TRUE); ;
    break;}
case 164:
#line 871 "parser.y"
{ yyval.tree = list_start (yyvsp[0].tree, TRUE); ;
    break;}
case 165:
#line 872 "parser.y"
{ yyval.tree = list_chain (yyvsp[-1].tree, yyvsp[0].tree, TRUE); ;
    break;}
case 166:
#line 876 "parser.y"
{
	yyval.tree = IDL_member_new (yyvsp[-2].tree, yyvsp[-1].tree);
	if (IDL_NODE_TYPE (yyvsp[-2].tree) == IDLN_IDENT &&
	    g_hash_table_lookup (struct_rdht, yyvsp[-2].tree)) {
		char *s = IDL_ns_ident_to_qstring (IDL_LIST (yyvsp[-1].tree).data, "::", 0);
		char *s2 = IDL_ns_ident_to_qstring (yyvsp[-2].tree, "::", 0);
		yyerrorv ("Member `%s' recurses structure `%s'", s, s2);
		free (s); free (s2);
	}
;
    break;}
case 180:
#line 907 "parser.y"
{ yyval.tree = IDL_type_sequence_new (yyvsp[-3].tree, yyvsp[-1].tree); ;
    break;}
case 181:
#line 910 "parser.y"
{ yyval.tree = IDL_type_sequence_new (yyvsp[-1].tree, NULL); ;
    break;}
case 182:
#line 913 "parser.y"
{ yyval.tree = IDL_type_float_new (IDL_FLOAT_TYPE_FLOAT); ;
    break;}
case 183:
#line 914 "parser.y"
{ yyval.tree = IDL_type_float_new (IDL_FLOAT_TYPE_DOUBLE); ;
    break;}
case 184:
#line 915 "parser.y"
{ yyval.tree = IDL_type_float_new (IDL_FLOAT_TYPE_LONGDOUBLE); ;
    break;}
case 185:
#line 920 "parser.y"
{ yyval.tree = IDL_type_fixed_new (yyvsp[-3].tree, yyvsp[-1].tree); ;
    break;}
case 186:
#line 923 "parser.y"
{ yyval.tree = IDL_type_fixed_new (NULL, NULL); ;
    break;}
case 187:
#line 926 "parser.y"
{ yyval.tree = IDL_type_integer_new (TRUE, yyvsp[0].integer); ;
    break;}
case 188:
#line 927 "parser.y"
{ yyval.tree = IDL_type_integer_new (FALSE, yyvsp[0].integer); ;
    break;}
case 189:
#line 930 "parser.y"
{ yyval.integer = IDL_INTEGER_TYPE_SHORT; ;
    break;}
case 190:
#line 931 "parser.y"
{ yyval.integer = IDL_INTEGER_TYPE_LONG; ;
    break;}
case 191:
#line 932 "parser.y"
{ yyval.integer = IDL_INTEGER_TYPE_LONGLONG; ;
    break;}
case 195:
#line 944 "parser.y"
{ yyval.integer = IDL_INTEGER_TYPE_SHORT; ;
    break;}
case 196:
#line 945 "parser.y"
{ yyval.integer = IDL_INTEGER_TYPE_LONG; ;
    break;}
case 197:
#line 946 "parser.y"
{ yyval.integer = IDL_INTEGER_TYPE_LONGLONG; ;
    break;}
case 201:
#line 958 "parser.y"
{ yyval.tree = IDL_type_char_new (); ;
    break;}
case 202:
#line 961 "parser.y"
{ yyval.tree = IDL_type_wide_char_new (); ;
    break;}
case 203:
#line 964 "parser.y"
{ yyval.tree = IDL_type_boolean_new (); ;
    break;}
case 204:
#line 967 "parser.y"
{ yyval.tree = IDL_type_octet_new (); ;
    break;}
case 205:
#line 970 "parser.y"
{ yyval.tree = IDL_type_any_new (); ;
    break;}
case 206:
#line 973 "parser.y"
{ yyval.tree = IDL_type_object_new (); ;
    break;}
case 207:
#line 976 "parser.y"
{ yyval.tree = IDL_type_typecode_new (); ;
    break;}
case 208:
#line 981 "parser.y"
{ yyval.tree = IDL_type_string_new (yyvsp[-1].tree); ;
    break;}
case 209:
#line 982 "parser.y"
{ yyval.tree = IDL_type_string_new (NULL); ;
    break;}
case 210:
#line 987 "parser.y"
{ yyval.tree = IDL_type_wide_string_new (yyvsp[-1].tree); ;
    break;}
case 211:
#line 988 "parser.y"
{ yyval.tree = IDL_type_wide_string_new (NULL); ;
    break;}
case 212:
#line 991 "parser.y"
{ yyval.tree = list_start (yyvsp[0].tree, TRUE); ;
    break;}
case 213:
#line 993 "parser.y"
{ yyval.tree = list_chain (yyvsp[-2].tree, yyvsp[0].tree, TRUE); ;
    break;}
case 218:
#line 1006 "parser.y"
{ yyval.tree = list_start (yyvsp[0].tree, TRUE); ;
    break;}
case 219:
#line 1008 "parser.y"
{ yyval.tree = list_chain (yyvsp[-2].tree, yyvsp[0].tree, TRUE); ;
    break;}
case 220:
#line 1012 "parser.y"
{ yyval.tree = IDL_type_array_new (yyvsp[-1].tree, yyvsp[0].tree); ;
    break;}
case 221:
#line 1015 "parser.y"
{ yyval.tree = list_start (yyvsp[0].tree, TRUE); ;
    break;}
case 222:
#line 1017 "parser.y"
{ yyval.tree = list_chain (yyvsp[-1].tree, yyvsp[0].tree, TRUE); ;
    break;}
case 223:
#line 1022 "parser.y"
{ yyval.tree = yyvsp[-1].tree; ;
    break;}
case 224:
#line 1026 "parser.y"
{
	yyval.hash_table = g_hash_table_new (IDL_strcase_hash, IDL_strcase_equal);
	g_hash_table_insert (yyval.hash_table, yyvsp[-1].str, yyvsp[0].str);
;
    break;}
case 225:
#line 1032 "parser.y"
{
	yyval.hash_table = yyvsp[-3].hash_table;
	g_hash_table_insert (yyval.hash_table, yyvsp[-1].str, yyvsp[0].str);
;
    break;}
case 226:
#line 1036 "parser.y"
{
	yyval.hash_table = g_hash_table_new (IDL_strcase_hash, IDL_strcase_equal);
	g_hash_table_insert (yyval.hash_table, yyvsp[0].str, g_strdup (""));
;
    break;}
case 227:
#line 1041 "parser.y"
{
	yyval.hash_table = yyvsp[-2].hash_table;
	g_hash_table_insert (yyval.hash_table, yyvsp[0].str, g_strdup (""));
;
    break;}
case 228:
#line 1047 "parser.y"
{ yyval.tree = IDL_ident_new (yyvsp[0].str); ;
    break;}
case 229:
#line 1050 "parser.y"
{
	assert (yyvsp[0].tree != NULL);
	assert (IDL_NODE_TYPE (yyvsp[0].tree) == IDLN_GENTREE);
	assert (IDL_NODE_TYPE (IDL_GENTREE (yyvsp[0].tree).data) == IDLN_IDENT);
	yyval.tree = IDL_GENTREE (yyvsp[0].tree).data;
;
    break;}
case 230:
#line 1058 "parser.y"
{
	IDL_ns_push_scope (__IDL_root_ns, yyvsp[0].tree);
#ifdef YYDEBUG
	if (yydebug)
		printf ("entering new/prev scope of %s\n", 
		       IDL_IDENT (IDL_GENTREE (IDL_NS (__IDL_root_ns).current).data).str);
#endif
	assert (IDL_NODE_TYPE (IDL_GENTREE (yyvsp[0].tree).data) == IDLN_IDENT);
	yyval.tree = IDL_GENTREE (yyvsp[0].tree).data;
;
    break;}
case 231:
#line 1070 "parser.y"
{
	IDL_ns_push_scope (__IDL_root_ns, yyvsp[0].tree);
	assert (IDL_NS (__IDL_root_ns).current != NULL);
	assert (IDL_NODE_TYPE (IDL_NS (__IDL_root_ns).current) == IDLN_GENTREE);
	assert (IDL_GENTREE (IDL_NS (__IDL_root_ns).current).data != NULL);
	assert (IDL_NODE_TYPE (IDL_GENTREE (IDL_NS (__IDL_root_ns).current).data) == IDLN_IDENT);
#ifdef YYDEBUG
	if (yydebug)
		printf ("entering new/prev scope of %s\n", 
		       IDL_IDENT (IDL_GENTREE (IDL_NS (__IDL_root_ns).current).data).str);
#endif
	assert (IDL_NODE_TYPE (IDL_GENTREE (yyvsp[0].tree).data) == IDLN_IDENT);
	yyval.tree = IDL_GENTREE (yyvsp[0].tree).data;
;
    break;}
case 232:
#line 1086 "parser.y"
{
#ifdef YYDEBUG
	if (yydebug)
		printf ("scope to parent of %s\n", 
		       IDL_IDENT (IDL_GENTREE (IDL_NS (__IDL_root_ns).current).data).str);
#endif
	IDL_ns_pop_scope (__IDL_root_ns);
;
    break;}
case 233:
#line 1096 "parser.y"
{
	IDL_tree p;

	if ((p = IDL_ns_place_new (__IDL_root_ns, yyvsp[0].tree)) == NULL) {
		IDL_tree q;
		int i;

		p = IDL_ns_lookup_cur_scope (__IDL_root_ns, yyvsp[0].tree, NULL);

		for (i = 0, q = IDL_GENTREE (p).data;
		     q && (IDL_NODE_TYPE (q) == IDLN_IDENT ||
			   IDL_NODE_TYPE (q) == IDLN_LIST) && i < 4;
		     ++i)
			if (IDL_NODE_UP (q))
				q = IDL_NODE_UP (q);

		if (q) {
			IDL_tree_error (yyvsp[0].tree, "`%s' conflicts", IDL_IDENT (yyvsp[0].tree).str);
			do_token_error (q, "with", FALSE);
		} else
			yyerrorv ("`%s' duplicate identifier", IDL_IDENT (yyvsp[0].tree).str);

		IDL_tree_free (yyvsp[0].tree);
		YYABORT;
	}
	assert (IDL_IDENT (yyvsp[0].tree)._ns_ref == p);
	++IDL_NODE_REFS (IDL_GENTREE (p).data);
	if (__IDL_new_ident_comments != NULL) {
		assert (IDL_IDENT (yyvsp[0].tree).comments == NULL);
		IDL_IDENT (yyvsp[0].tree).comments = __IDL_new_ident_comments;
		__IDL_new_ident_comments = NULL;
	}
	yyval.tree = p;
;
    break;}
case 234:
#line 1132 "parser.y"
{
	IDL_tree p;

	if ((p = IDL_ns_resolve_ident (__IDL_root_ns, yyvsp[0].tree)) == NULL) {
		yyerrorv ("`%s' undeclared identifier", IDL_IDENT (yyvsp[0].tree).str);
		IDL_tree_free (yyvsp[0].tree);
		YYABORT;
	}
	IDL_tree_free (yyvsp[0].tree);
	assert (IDL_GENTREE (p).data != NULL);
	assert (IDL_IDENT (IDL_GENTREE (p).data)._ns_ref == p);
	++IDL_NODE_REFS (IDL_GENTREE (p).data);
	yyval.tree = p;
;
    break;}
case 235:
#line 1149 "parser.y"
{
	IDL_tree p;

	if ((p = IDL_ns_lookup_cur_scope (__IDL_root_ns, yyvsp[0].tree, NULL)) == NULL) {
		p = IDL_ns_place_new (__IDL_root_ns, yyvsp[0].tree);
		assert (p != NULL);
		assert (IDL_IDENT (yyvsp[0].tree)._ns_ref == p);
		if (__IDL_new_ident_comments != NULL) {
			assert (IDL_IDENT (yyvsp[0].tree).comments == NULL);
			IDL_IDENT (yyvsp[0].tree).comments = __IDL_new_ident_comments;
			__IDL_new_ident_comments = NULL;
		}
	} else {
		IDL_tree_free (yyvsp[0].tree);
		assert (IDL_GENTREE (p).data != NULL);
		assert (IDL_IDENT (IDL_GENTREE (p).data)._ns_ref == p);
	}
	++IDL_NODE_REFS (IDL_GENTREE (p).data);
	yyval.tree = p;
;
    break;}
case 236:
#line 1171 "parser.y"
{
	IDL_tree p;

	if ((p = IDL_ns_lookup_this_scope (
		__IDL_root_ns,IDL_NS (__IDL_root_ns).file, yyvsp[0].tree, NULL)) == NULL) {
		yyerrorv ("`%s' undeclared identifier", IDL_IDENT (yyvsp[0].tree).str);
		IDL_tree_free (yyvsp[0].tree);
		YYABORT;
	}
	IDL_tree_free (yyvsp[0].tree);
	assert (IDL_GENTREE (p).data != NULL);
	assert (IDL_IDENT (IDL_GENTREE (p).data)._ns_ref == p);
	++IDL_NODE_REFS (IDL_GENTREE (p).data);
	yyval.tree = p;
;
    break;}
case 237:
#line 1188 "parser.y"
{ yyval.tree = list_start (yyvsp[0].tree, TRUE); ;
    break;}
case 238:
#line 1190 "parser.y"
{ yyval.tree = list_chain (yyvsp[-2].tree, yyvsp[0].tree, TRUE); ;
    break;}
case 239:
#line 1193 "parser.y"
{
	IDL_tree literal, ident = NULL;
	IDL_longlong_t value = 0;

	if ((literal = IDL_resolve_const_exp (yyvsp[0].tree, IDLN_INTEGER))) {
		assert (IDL_NODE_TYPE (literal) == IDLN_INTEGER);
		value = IDL_INTEGER (literal).value;
		IDL_tree_free (yyvsp[0].tree);
	}

	if (literal && IDL_NODE_UP (literal) &&
	    IDL_NODE_TYPE (IDL_NODE_UP (literal)) == IDLN_CONST_DCL)
		ident = IDL_CONST_DCL (IDL_NODE_UP (literal)).ident;
	
	if (literal == NULL) {
		if (!(__IDL_flags & IDLF_NO_EVAL_CONST))
			yyerror ("Could not resolve constant expression");
		yyval.tree = yyvsp[0].tree;
	} else if (value == 0) {
		yyerror ("Zero array size is illegal");
		if (ident)
			IDL_tree_error (ident, "From constant declared here");
		yyval.tree = NULL;
	} else if (value < 0) {
		yywarningv (IDL_WARNING1, "Cannot use negative value %"
			    IDL_LL "d, using %" IDL_LL "d",
			   value, -value);
		if (ident)
			IDL_tree_warning (ident,
					  IDL_WARNING1, "From constant declared here");
		yyval.tree = IDL_integer_new (-value);
	}
	else
		yyval.tree = IDL_integer_new (value);
;
    break;}
case 240:
#line 1230 "parser.y"
{ yyval.declspec = 0; ;
    break;}
case 241:
#line 1231 "parser.y"
{
	yyval.declspec = IDL_parse_declspec (yyvsp[0].str);
	free (yyvsp[0].str);
;
    break;}
case 242:
#line 1237 "parser.y"
{ yyval.hash_table = NULL; ;
    break;}
case 243:
#line 1238 "parser.y"
{
	/* Enable property scanning */
	if (__IDL_flags & IDLF_XPIDL)
		__IDL_flagsi |= IDLFP_XPIDL_PROPERTY;
	else {
		yyerror ("XPIDL syntax not enabled, cannot specify properties");
		YYABORT;
	}
;
    break;}
case 244:
#line 1247 "parser.y"
{
	/* Disable property scanning */
	if (__IDL_flags & IDLF_XPIDL)
		__IDL_flagsi &= ~IDLFP_XPIDL_PROPERTY;
	yyval.hash_table = yyvsp[-1].hash_table;
;
    break;}
case 245:
#line 1255 "parser.y"
{ yyval.tree = IDL_integer_new (yyvsp[0].integer); ;
    break;}
case 246:
#line 1258 "parser.y"
{ yyval.tree = IDL_string_new (yyvsp[0].str); ;
    break;}
case 247:
#line 1261 "parser.y"
{ yyval.tree = IDL_char_new (yyvsp[0].str); ;
    break;}
case 248:
#line 1264 "parser.y"
{ yyval.tree = IDL_fixed_new (yyvsp[0].str); ;
    break;}
case 249:
#line 1267 "parser.y"
{ yyval.tree = IDL_float_new (yyvsp[0].floatp); ;
    break;}
case 250:
#line 1270 "parser.y"
{ yyval.tree = IDL_boolean_new (TRUE); ;
    break;}
case 251:
#line 1271 "parser.y"
{ yyval.tree = IDL_boolean_new (FALSE); ;
    break;}
case 254:
#line 1278 "parser.y"
{
	char *catstr = (char *) malloc (strlen (yyvsp[-1].str) + strlen (yyvsp[0].str) + 1);
	strcpy (catstr, yyvsp[-1].str); free (yyvsp[-1].str);
	strcat (catstr, yyvsp[0].str); free (yyvsp[0].str);
	yyval.str = catstr;
;
    break;}
case 255:
#line 1286 "parser.y"
{
	char *s = IDL_do_escapes (yyvsp[0].str);
	free (yyvsp[0].str);
	yyval.str = s;
;
    break;}
case 256:
#line 1293 "parser.y"
{
	char *s = IDL_do_escapes (yyvsp[0].str);
	free (yyvsp[0].str);
	yyval.str = s;
;
    break;}
}
   /* the action file gets copied in in place of this dollarsign */
#line 498 "/usr/lib/bison.simple"

  yyvsp -= yylen;
  yyssp -= yylen;
#ifdef YYLSP_NEEDED
  yylsp -= yylen;
#endif

#if YYDEBUG != 0
  if (yydebug)
    {
      short *ssp1 = yyss - 1;
      fprintf (stderr, "state stack now");
      while (ssp1 != yyssp)
	fprintf (stderr, " %d", *++ssp1);
      fprintf (stderr, "\n");
    }
#endif

  *++yyvsp = yyval;

#ifdef YYLSP_NEEDED
  yylsp++;
  if (yylen == 0)
    {
      yylsp->first_line = yylloc.first_line;
      yylsp->first_column = yylloc.first_column;
      yylsp->last_line = (yylsp-1)->last_line;
      yylsp->last_column = (yylsp-1)->last_column;
      yylsp->text = 0;
    }
  else
    {
      yylsp->last_line = (yylsp+yylen-1)->last_line;
      yylsp->last_column = (yylsp+yylen-1)->last_column;
    }
#endif

  /* Now "shift" the result of the reduction.
     Determine what state that goes to,
     based on the state we popped back to
     and the rule number reduced by.  */

  yyn = yyr1[yyn];

  yystate = yypgoto[yyn - YYNTBASE] + *yyssp;
  if (yystate >= 0 && yystate <= YYLAST && yycheck[yystate] == *yyssp)
    yystate = yytable[yystate];
  else
    yystate = yydefgoto[yyn - YYNTBASE];

  goto yynewstate;

yyerrlab:   /* here on detecting error */

  if (! yyerrstatus)
    /* If not already recovering from an error, report this error.  */
    {
      ++yynerrs;

#ifdef YYERROR_VERBOSE
      yyn = yypact[yystate];

      if (yyn > YYFLAG && yyn < YYLAST)
	{
	  int size = 0;
	  char *msg;
	  int x, count;

	  count = 0;
	  /* Start X at -yyn if nec to avoid negative indexes in yycheck.  */
	  for (x = (yyn < 0 ? -yyn : 0);
	       x < (sizeof(yytname) / sizeof(char *)); x++)
	    if (yycheck[x + yyn] == x)
	      size += strlen(yytname[x]) + 15, count++;
	  msg = (char *) malloc(size + 15);
	  if (msg != 0)
	    {
	      strcpy(msg, "parse error");

	      if (count < 5)
		{
		  count = 0;
		  for (x = (yyn < 0 ? -yyn : 0);
		       x < (sizeof(yytname) / sizeof(char *)); x++)
		    if (yycheck[x + yyn] == x)
		      {
			strcat(msg, count == 0 ? ", expecting `" : " or `");
			strcat(msg, yytname[x]);
			strcat(msg, "'");
			count++;
		      }
		}
	      yyerror(msg);
	      free(msg);
	    }
	  else
	    yyerror ("parse error; also virtual memory exceeded");
	}
      else
#endif /* YYERROR_VERBOSE */
	yyerror("parse error");
    }

  goto yyerrlab1;
yyerrlab1:   /* here on error raised explicitly by an action */

  if (yyerrstatus == 3)
    {
      /* if just tried and failed to reuse lookahead token after an error, discard it.  */

      /* return failure if at end of input */
      if (yychar == YYEOF)
	YYABORT;

#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Discarding token %d (%s).\n", yychar, yytname[yychar1]);
#endif

      yychar = YYEMPTY;
    }

  /* Else will try to reuse lookahead token
     after shifting the error token.  */

  yyerrstatus = 3;		/* Each real token shifted decrements this */

  goto yyerrhandle;

yyerrdefault:  /* current state does not do anything special for the error token. */

#if 0
  /* This is wrong; only states that explicitly want error tokens
     should shift them.  */
  yyn = yydefact[yystate];  /* If its default is to accept any token, ok.  Otherwise pop it.*/
  if (yyn) goto yydefault;
#endif

yyerrpop:   /* pop the current state because it cannot handle the error token */

  if (yyssp == yyss) YYABORT;
  yyvsp--;
  yystate = *--yyssp;
#ifdef YYLSP_NEEDED
  yylsp--;
#endif

#if YYDEBUG != 0
  if (yydebug)
    {
      short *ssp1 = yyss - 1;
      fprintf (stderr, "Error: state stack now");
      while (ssp1 != yyssp)
	fprintf (stderr, " %d", *++ssp1);
      fprintf (stderr, "\n");
    }
#endif

yyerrhandle:

  yyn = yypact[yystate];
  if (yyn == YYFLAG)
    goto yyerrdefault;

  yyn += YYTERROR;
  if (yyn < 0 || yyn > YYLAST || yycheck[yyn] != YYTERROR)
    goto yyerrdefault;

  yyn = yytable[yyn];
  if (yyn < 0)
    {
      if (yyn == YYFLAG)
	goto yyerrpop;
      yyn = -yyn;
      goto yyreduce;
    }
  else if (yyn == 0)
    goto yyerrpop;

  if (yyn == YYFINAL)
    YYACCEPT;

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Shifting error token, ");
#endif

  *++yyvsp = yylval;
#ifdef YYLSP_NEEDED
  *++yylsp = yylloc;
#endif

  yystate = yyn;
  goto yynewstate;
}
#line 1300 "parser.y"


static const char *IDL_ns_get_cur_prefix (IDL_ns ns)
{
	IDL_tree p;

	p = IDL_NS (ns).current;

	assert (p != NULL);

	while (p && !IDL_GENTREE (p)._cur_prefix)
		p = IDL_NODE_UP (p);

	return p ? IDL_GENTREE (p)._cur_prefix : NULL;
}

char *IDL_ns_ident_make_repo_id (IDL_ns ns, IDL_tree p,
				 const char *p_prefix, int *major, int *minor)
{
	GString *s = g_string_new (NULL);
	const char *prefix;
	char *q;

	assert (p != NULL);
	
	if (IDL_NODE_TYPE (p) == IDLN_IDENT)
		p = IDL_IDENT_TO_NS (p);

	assert (p != NULL);

	prefix = p_prefix ? p_prefix : IDL_ns_get_cur_prefix (ns);

	q = IDL_ns_ident_to_qstring (p, "/", 0);
	g_string_sprintf (s, "IDL:%s%s%s:%d.%d",
			  prefix ? prefix : "",
			  prefix && *prefix ? "/" : "",
			  q,
			  major ? *major : 1,
			  minor ? *minor : 0);
	free (q);

	q = s->str;
	g_string_free (s, FALSE);

	return q;
}

static const char *get_name_token (const char *s, char **tok)
{
	const char *begin = s;
	int state = 0;

	if (!s)
		return NULL;

	while (isspace (*s)) ++s;
	
	while (1) switch (state) {
	case 0:		/* Unknown */
		if (*s == ':')
			state = 1;
		else if (isalnum (*s) || *s == '_') {
			begin = s;
			state = 2;
		} else
			return NULL;
		break;
	case 1:		/* Scope */
		if (strncmp (s, "::", 2) == 0) {
			char *r = (char *) malloc (3);
			strcpy (r, "::");
			*tok = r;
			return s + 2;
		} else	/* Invalid */
			return NULL;
		break;
	case 2:
		if (isalnum (*s) || *s == '_')
			++s;
		else {
			char *r = (char *) malloc (s - begin + 1);
			strncpy (r, begin, s - begin + 1);
			r[s - begin] = 0;
			*tok = r;
			return s;
		}
		break;
	}
}

static IDL_tree IDL_ns_pragma_parse_name (IDL_ns ns, const char *s)
{
	IDL_tree p = IDL_NS (ns).current;
	int start = 1;
	char *tok;

	while (p && *s && (s = get_name_token (s, &tok))) {
		if (tok == NULL)
			return NULL;
		if (strcmp (tok, "::") == 0) {
			if (start) {
				/* Globally scoped */
				p = IDL_NS (ns).file;
			}
			free (tok);
		} else {
			IDL_tree ident = IDL_ident_new (tok);
			p = IDL_ns_lookup_this_scope (__IDL_root_ns, p, ident, NULL);
			IDL_tree_free (ident);
		}
		start = 0;
	}
	
	return p;
}

void IDL_ns_ID (IDL_ns ns, const char *s)
{
	char name[1024], id[1024];
	IDL_tree p, ident;
	int n;

	n = sscanf (s, "%1023s \"%1023s\"", name, id);
	if (n < 2 && __IDL_is_parsing) {
		yywarning (IDL_WARNING1, "Malformed pragma ID");
		return;
	}
	if (id[strlen (id) - 1] == '"')
		id[strlen (id) - 1] = 0;

	p = IDL_ns_pragma_parse_name (__IDL_root_ns, name);
	if (!p && __IDL_is_parsing) {
		yywarningv (IDL_WARNING1, "Unknown identifier `%s' in pragma ID", name);
		return;
	}

	/* We have resolved the identifier, so assign the repo id */
	assert (IDL_NODE_TYPE (p) == IDLN_GENTREE);
	assert (IDL_GENTREE (p).data != NULL);
	assert (IDL_NODE_TYPE (IDL_GENTREE (p).data) == IDLN_IDENT);
	ident = IDL_GENTREE (p).data;

	if (IDL_IDENT_REPO_ID (ident) != NULL)
		free (IDL_IDENT_REPO_ID (ident));

	IDL_IDENT_REPO_ID (ident) = g_strdup (id);
}

void IDL_ns_version (IDL_ns ns, const char *s)
{
	char name[1024];
	int n, major, minor;
	IDL_tree p, ident;

	n = sscanf (s, "%1023s %u %u", name, &major, &minor);
	if (n < 3 && __IDL_is_parsing) {
		yywarning (IDL_WARNING1, "Malformed pragma version");
		return;
	}

	p = IDL_ns_pragma_parse_name (__IDL_root_ns, name);
	if (!p && __IDL_is_parsing) {
		yywarningv (IDL_WARNING1, "Unknown identifier `%s' in pragma version", name);
		return;
	}

	/* We have resolved the identifier, so assign the repo id */
	assert (IDL_NODE_TYPE (p) == IDLN_GENTREE);
	assert (IDL_GENTREE (p).data != NULL);
	assert (IDL_NODE_TYPE (IDL_GENTREE (p).data) == IDLN_IDENT);
	ident = IDL_GENTREE (p).data;

	if (IDL_IDENT_REPO_ID (ident) != NULL) {
		char *v = strrchr (IDL_IDENT_REPO_ID (ident), ':');
		if (v) {
			GString *s;

			*v = 0;
			s = g_string_new (NULL);
			g_string_sprintf (s, "%s:%d.%d",
					  IDL_IDENT_REPO_ID (ident), major, minor);
			free (IDL_IDENT_REPO_ID (ident));
			IDL_IDENT_REPO_ID (ident) = s->str;
			g_string_free (s, FALSE);
		} else if (__IDL_is_parsing)
			yywarningv (IDL_WARNING1, "Cannot find RepositoryID OMG IDL version in ID `%s'",
				    IDL_IDENT_REPO_ID (ident));
	} else
		IDL_IDENT_REPO_ID (ident) =
			IDL_ns_ident_make_repo_id (
				__IDL_root_ns, p, NULL, &major, &minor);
}

int IDL_inhibit_get (void)
{
	g_return_val_if_fail (__IDL_is_parsing, -1);

	return __IDL_inhibits;
}

void IDL_inhibit_push (void)
{
	g_return_if_fail (__IDL_is_parsing);

	++__IDL_inhibits;
}

void IDL_inhibit_pop (void)
{
	g_return_if_fail (__IDL_is_parsing);

	if (--__IDL_inhibits < 0)
		__IDL_inhibits = 0;
}

static void IDL_inhibit (IDL_ns ns, const char *s)
{
	if (g_strcasecmp ("push", s) == 0)
		IDL_inhibit_push ();
	else if (g_strcasecmp ("pop", s) == 0)
		IDL_inhibit_pop ();
}

void __IDL_do_pragma (const char *s)
{
	int n;
	char directive[256];

	g_return_if_fail (__IDL_is_parsing);
	g_return_if_fail (s != NULL);

	if (sscanf (s, "%255s%n", directive, &n) < 1)
		return;
	s += n;
	while (isspace (*s)) ++s;

	if (strcmp (directive, "prefix") == 0)
		IDL_ns_prefix (__IDL_root_ns, s);
	else if (strcmp (directive, "ID") == 0)
		IDL_ns_ID (__IDL_root_ns, s);
	else if (strcmp (directive, "version") == 0)
		IDL_ns_version (__IDL_root_ns, s);
	else if (strcmp (directive, "inhibit") == 0)
		IDL_inhibit (__IDL_root_ns, s);
}

static IDL_declspec_t IDL_parse_declspec (const char *strspec)
{
	IDL_declspec_t flags = IDLF_DECLSPEC_EXIST;

	if (strspec == NULL)
		return flags;

	if (strcmp (strspec, "inhibit") == 0)
		flags |= IDLF_DECLSPEC_INHIBIT;
	else if (__IDL_is_parsing)
		yywarningv (IDL_WARNING1, "Ignoring unknown declspec `%s'", strspec);

	return flags;
}

void IDL_file_set (const char *filename, int line)
{
	IDL_fileinfo *fi;
	char *orig;

	g_return_if_fail (__IDL_is_parsing);

	if (filename) {
		__IDL_cur_filename = g_strdup (filename);

		if (
#ifdef HAVE_CPP_PIPE_STDIN
			!strlen (__IDL_cur_filename)
#else
			!strcmp (__IDL_cur_filename, __IDL_tmp_filename)
#endif
			) {
			free (__IDL_cur_filename);
			__IDL_cur_filename = g_strdup (__IDL_real_filename);
		}

		if (g_hash_table_lookup_extended (__IDL_filename_hash, __IDL_cur_filename,
						  (gpointer) &orig, (gpointer) &fi)) {
			free (__IDL_cur_filename);
			__IDL_cur_filename = orig;
			__IDL_cur_fileinfo = fi;
		} else {
			fi = g_new0 (IDL_fileinfo, 1);
			__IDL_cur_fileinfo = fi;
			g_hash_table_insert (__IDL_filename_hash, __IDL_cur_filename, fi);
		}
	}

	if (__IDL_cur_line > 0)
		__IDL_cur_line = line;
}

void IDL_file_get (const char **filename, int *line)
{
	g_return_if_fail (__IDL_is_parsing);

	if (filename)
		*filename = __IDL_cur_filename;

	if (line)
		*line = __IDL_cur_line;
}

static int do_token_error (IDL_tree p, const char *message, gboolean prev)
{
	int dienow;
	char *what = NULL, *who = NULL;

	assert (p != NULL);

	dienow = IDL_tree_get_node_info (p, &what, &who);

	assert (what != NULL);
	
	if (who && *who)
		IDL_tree_error (p, "%s %s `%s'", message, what, who);
	else
		IDL_tree_error (p, "%s %s", message, what);
	
	return dienow;
}

static IDL_tree list_start (IDL_tree a, gboolean filter_null)
{
	IDL_tree p;

	if (!a && filter_null)
		return NULL;

	p = IDL_list_new (a);

	return p;
}

static IDL_tree list_chain (IDL_tree a, IDL_tree b, gboolean filter_null)
{
	IDL_tree p;

	if (filter_null) {
		if (!b)
			return a;
		
		if (!a)
			return list_start (b, filter_null);
	}

	p = IDL_list_new (b);
	a = IDL_list_concat (a, p);

	return a;
}

static IDL_tree zlist_chain (IDL_tree a, IDL_tree b, gboolean filter_null)
{
	if (a == NULL)
		return list_start (b, filter_null);
	else
		return list_chain (a, b, filter_null);
}

static int IDL_binop_chktypes (enum IDL_binop op, IDL_tree a, IDL_tree b)
{
	if (IDL_NODE_TYPE (a) != IDLN_BINOP &&
	    IDL_NODE_TYPE (b) != IDLN_BINOP &&
	    IDL_NODE_TYPE (a) != IDLN_UNARYOP &&
	    IDL_NODE_TYPE (b) != IDLN_UNARYOP &&
	    IDL_NODE_TYPE (a) != IDL_NODE_TYPE (b)) {
		yyerror ("Invalid mix of types in constant expression");
		return -1;
	}

	switch (op) {
	case IDL_BINOP_MULT:
	case IDL_BINOP_DIV:
	case IDL_BINOP_ADD:
	case IDL_BINOP_SUB:
		break;

	case IDL_BINOP_MOD:
	case IDL_BINOP_SHR:
	case IDL_BINOP_SHL:
	case IDL_BINOP_AND:
	case IDL_BINOP_OR:
	case IDL_BINOP_XOR:
		if ((IDL_NODE_TYPE (a) != IDLN_INTEGER ||
		     IDL_NODE_TYPE (b) != IDLN_INTEGER) &&
		    !(IDL_NODE_TYPE (a) == IDLN_BINOP ||
		      IDL_NODE_TYPE (b) == IDLN_BINOP ||
		      IDL_NODE_TYPE (a) == IDLN_UNARYOP ||
		      IDL_NODE_TYPE (b) == IDLN_UNARYOP)) {
			yyerror ("Invalid operation on non-integer value");
			return -1;
		}
		break;
	}

	return 0;
}

static int IDL_unaryop_chktypes (enum IDL_unaryop op, IDL_tree a)
{
	switch (op) {
	case IDL_UNARYOP_PLUS:
	case IDL_UNARYOP_MINUS:
		break;

	case IDL_UNARYOP_COMPLEMENT:
		if (IDL_NODE_TYPE (a) != IDLN_INTEGER &&
		    !(IDL_NODE_TYPE (a) == IDLN_BINOP ||
		      IDL_NODE_TYPE (a) == IDLN_UNARYOP)) {
			yyerror ("Operand to complement must be integer");
			return -1;
		}
		break;
	}

	return 0;
}

static IDL_tree IDL_binop_eval_integer (enum IDL_binop op, IDL_tree a, IDL_tree b)
{
	IDL_tree p = NULL;

	assert (IDL_NODE_TYPE (a) == IDLN_INTEGER);

	switch (op) {
	case IDL_BINOP_MULT:
		p = IDL_integer_new (IDL_INTEGER (a).value * IDL_INTEGER (b).value);
		break;

	case IDL_BINOP_DIV:
		if (IDL_INTEGER (b).value == 0) {
			yyerror ("Divide by zero in constant expression");
			return NULL;
		}
		p = IDL_integer_new (IDL_INTEGER (a).value / IDL_INTEGER (b).value);
		break;

	case IDL_BINOP_ADD:
		p = IDL_integer_new (IDL_INTEGER (a).value + IDL_INTEGER (b).value);
		break;

	case IDL_BINOP_SUB:
		p = IDL_integer_new (IDL_INTEGER (a).value - IDL_INTEGER (b).value);
		break;

	case IDL_BINOP_MOD:
		if (IDL_INTEGER (b).value == 0) {
			yyerror ("Modulo by zero in constant expression");
			return NULL;
		}
		p = IDL_integer_new (IDL_INTEGER (a).value % IDL_INTEGER (b).value);
		break;

	case IDL_BINOP_SHR:
		p = IDL_integer_new (IDL_INTEGER (a).value >> IDL_INTEGER (b).value);
		break;

	case IDL_BINOP_SHL:
		p = IDL_integer_new (IDL_INTEGER (a).value << IDL_INTEGER (b).value);
		break;

	case IDL_BINOP_AND:
		p = IDL_integer_new (IDL_INTEGER (a).value & IDL_INTEGER (b).value);
		break;

	case IDL_BINOP_OR:
		p = IDL_integer_new (IDL_INTEGER (a).value | IDL_INTEGER (b).value);
		break;

	case IDL_BINOP_XOR:
		p = IDL_integer_new (IDL_INTEGER (a).value ^ IDL_INTEGER (b).value);
		break;
	}

	return p;
}

static IDL_tree IDL_binop_eval_float (enum IDL_binop op, IDL_tree a, IDL_tree b)
{
	IDL_tree p = NULL;

	assert (IDL_NODE_TYPE (a) == IDLN_FLOAT);

	switch (op) {
	case IDL_BINOP_MULT:
		p = IDL_float_new (IDL_FLOAT (a).value * IDL_FLOAT (b).value);
		break;

	case IDL_BINOP_DIV:
		if (IDL_FLOAT (b).value == 0.0) {
			yyerror ("Divide by zero in constant expression");
			return NULL;
		}
		p = IDL_float_new (IDL_FLOAT (a).value / IDL_FLOAT (b).value);
		break;

	case IDL_BINOP_ADD:
		p = IDL_float_new (IDL_FLOAT (a).value + IDL_FLOAT (b).value);
		break;

	case IDL_BINOP_SUB:
		p = IDL_float_new (IDL_FLOAT (a).value - IDL_FLOAT (b).value);
		break;

	default:
		break;
	}

	return p;
}

static IDL_tree IDL_binop_eval (enum IDL_binop op, IDL_tree a, IDL_tree b)
{
	assert (IDL_NODE_TYPE (a) == IDL_NODE_TYPE (b));

	switch (IDL_NODE_TYPE (a)) {
	case IDLN_INTEGER: return IDL_binop_eval_integer (op, a, b);
	case IDLN_FLOAT: return IDL_binop_eval_float (op, a, b);
	default: return NULL;
	}
}

static IDL_tree IDL_unaryop_eval_integer (enum IDL_unaryop op, IDL_tree a)
{
	IDL_tree p = NULL;

	assert (IDL_NODE_TYPE (a) == IDLN_INTEGER);

	switch (op) {
	case IDL_UNARYOP_PLUS:
		p = IDL_integer_new (IDL_INTEGER (a).value);
		break;

	case IDL_UNARYOP_MINUS:
		p = IDL_integer_new (-IDL_INTEGER (a).value);
		break;

	case IDL_UNARYOP_COMPLEMENT:
		p = IDL_integer_new (~IDL_INTEGER (a).value);
		break;
	}
       
	return p;
}

static IDL_tree IDL_unaryop_eval_fixed (enum IDL_unaryop op, IDL_tree a)
{
	IDL_tree p = NULL;

	assert (IDL_NODE_TYPE (a) == IDLN_FIXED);

	switch (op) {
	case IDL_UNARYOP_PLUS:
		p = IDL_fixed_new (IDL_FIXED (a).value);
		break;

	default:
		break;
	}
       
	return p;
}

static IDL_tree IDL_unaryop_eval_float (enum IDL_unaryop op, IDL_tree a)
{
	IDL_tree p = NULL;

	assert (IDL_NODE_TYPE (a) == IDLN_FLOAT);

	switch (op) {
	case IDL_UNARYOP_PLUS:
		p = IDL_float_new (IDL_FLOAT (a).value);
		break;

	case IDL_UNARYOP_MINUS:
		p = IDL_float_new (-IDL_FLOAT (a).value);
		break;

	default:
		break;
	}
       
	return p;
}

static IDL_tree IDL_unaryop_eval (enum IDL_unaryop op, IDL_tree a)
{
	switch (IDL_NODE_TYPE (a)) {
	case IDLN_INTEGER: return IDL_unaryop_eval_integer (op, a);
	case IDLN_FIXED: return IDL_unaryop_eval_fixed (op, a);
	case IDLN_FLOAT: return IDL_unaryop_eval_float (op, a);
	default: return NULL;
	}
}

IDL_tree IDL_resolve_const_exp (IDL_tree p, IDL_tree_type type)
{
	gboolean resolved_value = FALSE, die = FALSE;
	gboolean wrong_type = FALSE;

	while (!resolved_value && !die) {
		if (IDL_NODE_TYPE (p) == IDLN_IDENT) {
			IDL_tree q = IDL_NODE_UP (p);
			
			assert (q != NULL);
			if (IDL_NODE_UP (q) &&
			    IDL_NODE_TYPE (IDL_NODE_UP (q)) == IDLN_TYPE_ENUM) {
				p = q;
				die = TRUE;
				break;
			} else if (IDL_NODE_TYPE (q) != IDLN_CONST_DCL) {
				p = q;
				wrong_type = TRUE;
				die = TRUE;
			} else
 				p = IDL_CONST_DCL (q).const_exp;
		}
		
		if (p == NULL ||
		    IDL_NODE_TYPE (p) == IDLN_BINOP ||
		    IDL_NODE_TYPE (p) == IDLN_UNARYOP) {
			die = TRUE;
			continue;
		}
		
		resolved_value = IDL_NODE_IS_LITERAL (p);
	}

	if (resolved_value &&
	    type != IDLN_ANY &&
	    IDL_NODE_TYPE (p) != type)
		wrong_type = TRUE;
	
	if (wrong_type) {
		yyerror ("Invalid type for constant");
		IDL_tree_error (p, "Previous resolved type declaration");
		return NULL;
	}

	return resolved_value ? p : NULL;
}

void IDL_queue_new_ident_comment (const char *str)
{
	g_return_if_fail (str != NULL);

	__IDL_new_ident_comments = g_slist_append (__IDL_new_ident_comments, g_strdup (str));
}

/*
 * Local variables:
 * mode: C
 * c-basic-offset: 8
 * tab-width: 8
 * indent-tabs-mode: t
 * End:
 */
